/*

  COPYRIGHT
  --------
  
    Copyright Maurizio Colucci -  9 nov 2003.
  
  COPYING
  -------
  
    This file is part of segusoLand.

    segusoLand is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    segusoLand is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with segusoLand; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  ABOUT THE AUTHOR
  ----------------

  You can contact me at seguso.forever@tin.it

  I live in Pescara, Italy.

  Job offers inside and outside Italy are very welcome :-)

 QUICK INTRO TO THE CODE
 -----------------------

*  If you are looking for the main() procedure, it is 
   run/1, further down this file. Actually, run/1 is called by

      int main(void)

   which is written in C++, in the file segusoLand.cc.

*  if you can't find the definition of some predicates, it's because
   they are written in C. This was necessary in order to use GTK+.
   For example, widgetShow. See segusoLand.cc.

*/

:-dynamic(selected/2). % Kind, object, 
:-dynamic(curTable/2). % used globally
:-dynamic(recentView/3). %recentView(Kind, Predicate, ReadableNameOfPredicate).
:-dynamic(table/3).
:-dynamic(showHiddenFiles/0).
:-dynamic(t/3). % example: t(file, 'good', 'doc.pdf').   t(program, editor, emacs).
:-dynamic(ext/2). % example: ext(png, '/usr/doc.png').
:-dynamic(hiddenFile/1).
:-dynamic(segusoDir/1). % segusoDir(D) means that D is a directory indexed by segusoLand

%@@fix bug when I rename to a file that already exists

% REGION table/3 is a table containing all the tables. Just like mySql :-)
table(file, 'Files to dispatch', real).
table(program, 'All programs', real).
table(file, 'All files', real).
table(verb, 'All verbs', real).
table(device, 'All devices', real).
table(time, 'All times', real).
% END

curTable(file, 'All files'). % this means that by default I will display each file X such that t(file, 'All files', X).
curTable(verb, 'All verbs').
curTable(device, 'All devices').
curTable(program, 'All programs').
curTable(time, 'All times').

predefinedTable(file, toDispatch, 'Files to dispatch').
predefinedTable(verb, toDispatch, 'Verbs to dispatch').
predefinedTable(device, toDispatch, 'Devices to dispatch').
predefinedTable(program, toDispatch, 'Programs to dispatch').
predefinedTable(file, all, 'All files').
predefinedTable(verb, all, 'All verbs').
predefinedTable(device, all, 'All devices').
predefinedTable(program, all, 'All programs').


remove(Item, List, NewList):-
	remove(Item, List, [], NewList).
remove(_, [], CurrentList, CurrentList).
remove(Item, List, CurrentList, NewList):-
	List\=[],
	List=[Item|Tail],
	remove(Item, Tail, CurrentList, NewList).
remove(Item, List, CurrentList, NewList):-
	List\=[],
	List=[H|Tail],
	H\=Item,
	remove(Item, Tail, [H|CurrentList], NewList).




selectedFiles(L):-
	findall(X, selected(file, X), L).

selectedTimes(L):-
	findall(X, selected(time, X), L).

theVerbIsVisible(V):-
	selected(verb, V). % if it is selected, it is the ONLY visible verb, so cut the rest.
theVerbIsVisible(V):-
	\+selected(verb, _),
	curTable(verb, T),
	t(T,verb, V),
	theNameMatchesFilter(verb, V),
	\+selectionIncompatibleWithVerb(V).
selectionIncompatibleWithVerb(V):-
	% find all the types required by the verb. 
	findall(verbRequiresAtLeastOneObjectSatisfying(V, K, P),
		verbRequiresAtLeastOneObjectSatisfying(V, K, P), Predicates),
	selected(K2, IncompatibleMember),
	K2\=time,
	K2\=verb,
	forall(member(verbRequiresAtLeastOneObjectSatisfying(V, K3, P3), Predicates), %...and is not required
	       (K3\=K2 ; \+call(P3, IncompatibleMember))).  % it is the same as  \+Q(IncompatibleMember)
selectionIncompatibleWithVerb('Rename'):-
	selectedFiles(L),
	length(L, Len),
	Len>1.

selectionIncompatibleWithVerb('Rename multiple files'):-
	selectedFiles(L),
	length(L, 1).

/*
selectionIncompatibleWithVerb('Play movie with subtitle file', L):-
	pair(X, Y, L),
	subtitleFile(X),
	subtitleFile(Y).
selectionIncompatibleWithVerb('Play movie with subtitle file', L):-
	pair(X, Y, L),
	videoFile(X),
	videoFile(Y).
selectionIncompatibleWithVerb('Burn data CD', L):-
	pair(X, Y, L),
	supports( 'Burn data CD', X),
	supports( 'Burn data CD', Y).
selectionIncompatibleWithVerb('Burn from ISO', L):-
	pair(M,N, L),
	cdBurningDevice(M),
	cdBurningDevice(N).
selectionIncompatibleWithVerb('Burn from ISO', L):-
	pair(M, N, L),
	supports( 'Burn from ISO', M),
	supports( 'Burn from ISO', N).
selectionIncompatibleWithVerb('Burn from ISO', L):-
	pair(M, N, L),
	holds(iso, M),
	holds(iso, N).
selectionIncompatibleWithVerb('Burn audio CD', L):-
	pair(M, N, L),
	supports( 'Burn audio CD', M), 
	supports( 'Burn audio CD', N).
selectionIncompatibleWithVerb('Burn audio CD', L):-
	pair(M, N, L),
	cdBurningDevice(M),
	cdBurningDevice(N).
selectionIncompatibleWithVerb('Burn video CD', L):-
	pair(M, N, L),
	supports( 'Burn video CD', M), 
	supports( 'Burn video CD', N).
selectionIncompatibleWithVerb('Print text file', L):-
	pair(X, Y, L),
	supports( 'Print text file', X),
	supports( 'Print text file', Y).
selectionIncompatibleWithVerb('Print PDF', L):-
	pair(X, Y, L),
	supports( 'Print PDF', X),
	supports( 'Print PDF', Y).
selectionIncompatibleWithVerb('Burn CD from BIN and CUE files', L):-
	pair(M, N, L),
	cdBurningDevice(M),
	cdBurningDevice(N).
selectionIncompatibleWithVerb('Burn CD from BIN and CUE files', L):-
	pair(M, N, L),
	supports(  'Burn CD from BIN and CUE files', M),
	supports(  'Burn CD from BIN and CUE files', N).
selectionIncompatibleWithVerb('Burn CD from BIN and CUE files', L):-
	pair(M, N, L),
	bin(M),
	bin(N).
selectionIncompatibleWithVerb('Burn CD from BIN and CUE files', L):-
	pair(M, N, L),
	cue(M),
	cue(N).
selectionIncompatibleWithVerb('Burn CD from audio and CUE files', L):-
	pair(M, N, L),
	cdBurningDevice(M),
	cdBurningDevice(N).
selectionIncompatibleWithVerb('Burn CD from audio and CUE files', L):-
	pair(M, N, L),
	supports(  'Burn CD from audio and CUE files', M),
	supports(  'Burn CD from audio and CUE files', N).
selectionIncompatibleWithVerb('Burn CD from audio and CUE files', L):-
	pair(M, N, L),
	audioFile(M),
	audioFile(N).
selectionIncompatibleWithVerb('Burn CD from audio and CUE files', L):-
	pair(M, N, L),
	cue(M),
	cue(N).
selectionIncompatibleWithVerb('Print picture', L):-
	pair(M, N, L),
	supports( 'Print picture', M),
	supports( 'Print picture', N).
selectionIncompatibleWithVerb('Extract archive', L):-
	pair(M, N, L),
	supports( 'Extract archive', M),
	supports( 'Extract archive', N).
selectionIncompatibleWithVerb(createNewTextFile, L):-
	pair(M, N, L),
	writableDisk(M),
	writableDisk(N).
selectionIncompatibleWithVerb('Create new SXW file', L):-
	pair(M, N, L),
	writableDisk(M),
	writableDisk(N).
*/



selectionIncompatibleWithVerb('Add to different archives'):-
	findall(M, selected(file, M), List),
	length(List, 1).



verbRequiresAtLeastOneObjectSatisfying('Share and download files over the internet', program, supports('Share and download files over the internet')).

verbRequiresAtLeastOneObjectSatisfying('Rip CD', program, supports('Rip CD')).
verbRequiresAtLeastOneObjectSatisfying('Rip CD', device, cdDevice).


verbRequiresAtLeastOneObjectSatisfying('Rip DVD', program, supports('Rip DVD')).
verbRequiresAtLeastOneObjectSatisfying('Rip DVD', device, dvdDevice).

verbRequiresAtLeastOneObjectSatisfying('Close program automatically X minutes from now', program, anyProgram).

verbRequiresAtLeastOneObjectSatisfying('Browse the directory of this file', program, supports('Browse the directory of this file')).
verbRequiresAtLeastOneObjectSatisfying('Browse the directory of this file', file, anyFile).

verbRequiresAtLeastOneObjectSatisfying('View free space on disk', device, writableDisk).
verbRequiresAtLeastOneObjectSatisfying('View free space on disk', program, supports('View free space on disk')).

verbRequiresAtLeastOneObjectSatisfying('Partition disk', device, writableDisk).
verbRequiresAtLeastOneObjectSatisfying('Partition disk', program, supports('Partition disk')).

verbRequiresAtLeastOneObjectSatisfying('Create thumbnails', program, supports('Create thumbnails')).

verbRequiresAtLeastOneObjectSatisfying('Setup mouse', device, mouseDevice).

verbRequiresAtLeastOneObjectSatisfying('Read newsgroups', program,  supports('Read newsgroups')).

verbRequiresAtLeastOneObjectSatisfying('Download a file from the internet with resume',program,  supports('Download a file from the internet with resume')).

verbRequiresAtLeastOneObjectSatisfying('Take a snapshot of your screen', program, supports('Take a snapshot of your screen')).  

verbRequiresAtLeastOneObjectSatisfying('Send to Konqueror', program, supports('Send to Konqueror')).

verbRequiresAtLeastOneObjectSatisfying('Send to Konqueror', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Copy to ...', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Move to...', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Rename multiple files', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Install RPM', file,  ext(rpm)). % @@ slow? It scans linearly all rpms in the computer...
verbRequiresAtLeastOneObjectSatisfying('Install DEB', file,  ext(deb)).

verbRequiresAtLeastOneObjectSatisfying('View calendar and schedule', program, supports('View calendar and schedule')).

verbRequiresAtLeastOneObjectSatisfying('Eject disk',  device, ejectableDevice).

verbRequiresAtLeastOneObjectSatisfying('Create new text file',program,  supports('Create new text file')).

verbRequiresAtLeastOneObjectSatisfying('Create new PDF file', program, supports('Create new PDF file')).


verbRequiresAtLeastOneObjectSatisfying('Delete', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Rename', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('Format disk', device, writableDisk).

verbRequiresAtLeastOneObjectSatisfying('Play video',  file, videoFile).
verbRequiresAtLeastOneObjectSatisfying('Play video', program, supports('Play video')).

verbRequiresAtLeastOneObjectSatisfying('Play movie with subtitle file', file,  videoFile).
verbRequiresAtLeastOneObjectSatisfying('Play movie with subtitle file', program, supports('Play movie with subtitle file')).
verbRequiresAtLeastOneObjectSatisfying('Play movie with subtitle file', file,  subtitleFile).

verbRequiresAtLeastOneObjectSatisfying('Play audio', file,  audioFile).
verbRequiresAtLeastOneObjectSatisfying('Play audio', program, supports('Play audio')).

verbRequiresAtLeastOneObjectSatisfying('Enqueue audio for playing', file,  audioFile).
verbRequiresAtLeastOneObjectSatisfying('Enqueue audio for playing', program, supports('Enqueue audio for playing')).

verbRequiresAtLeastOneObjectSatisfying('Add to single archive',  file, anyFile).
verbRequiresAtLeastOneObjectSatisfying('Add to single archive', program, supports('Add to single archive')).

verbRequiresAtLeastOneObjectSatisfying('Add to different archives', file,  anyFile).
verbRequiresAtLeastOneObjectSatisfying('Add to different archives', program, supports('Add to different archives')).

verbRequiresAtLeastOneObjectSatisfying('Extract archive',  file, compressedFile).
verbRequiresAtLeastOneObjectSatisfying('Extract archive', program, supports('Extract archive')).

verbRequiresAtLeastOneObjectSatisfying('View content of archive',  file, compressedFile).
verbRequiresAtLeastOneObjectSatisfying('View content of archive', program, supports('View content of archive')).

verbRequiresAtLeastOneObjectSatisfying('Burn data CD', file,  anyFile).
verbRequiresAtLeastOneObjectSatisfying('Burn data CD', program, supports('Burn data CD')).
verbRequiresAtLeastOneObjectSatisfying('Burn data CD', device, cdBurningDevice).

verbRequiresAtLeastOneObjectSatisfying('Erase CD-RW', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Erase CD-RW', program, supports('Erase CD-RW')).

verbRequiresAtLeastOneObjectSatisfying('View PDF', file,  ext(pdf)).
verbRequiresAtLeastOneObjectSatisfying('View PDF', program, supports('View PDF')).

verbRequiresAtLeastOneObjectSatisfying('Burn from ISO', program, supports('Burn from ISO')).
verbRequiresAtLeastOneObjectSatisfying('Burn from ISO', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Burn from ISO', file,  ext(iso)).

verbRequiresAtLeastOneObjectSatisfying('Burn audio CD', file,  audioFile).
verbRequiresAtLeastOneObjectSatisfying('Burn audio CD', program, supports('Burn audio CD')).
verbRequiresAtLeastOneObjectSatisfying('Burn audio CD', device, cdBurningDevice).

verbRequiresAtLeastOneObjectSatisfying('Burn video CD',  file, videoFile).
verbRequiresAtLeastOneObjectSatisfying('Burn video CD', program, supports('Burn video CD')).
verbRequiresAtLeastOneObjectSatisfying('Burn video CD', device, cdBurningDevice).

verbRequiresAtLeastOneObjectSatisfying('Print text file',  file, plainTextFile).
verbRequiresAtLeastOneObjectSatisfying('Print text file', program, supports('Print text file')).
verbRequiresAtLeastOneObjectSatisfying('Print text file', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('Print PDF',  file, ext(pdf)).
verbRequiresAtLeastOneObjectSatisfying('Print PDF', program, supports('Print PDF')).
verbRequiresAtLeastOneObjectSatisfying('Print PDF', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files',  file, ext(cue)).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files',  file, ext(bin)).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from BIN and CUE files', program, supports('Burn CD from BIN and CUE files')).

verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files',  file, ext(cue)).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files',  file, audioFile).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files', device, cdBurningDevice).
verbRequiresAtLeastOneObjectSatisfying('Burn CD from audio and CUE files', program, supports('Burn CD from audio and CUE files')).

verbRequiresAtLeastOneObjectSatisfying('Check new email', program, supports('Check new email')).

verbRequiresAtLeastOneObjectSatisfying('Configure printer', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('Change keyboard layout', device, keyboardDevice).

verbRequiresAtLeastOneObjectSatisfying('Set sound volume', device, audioCardDevice).
verbRequiresAtLeastOneObjectSatisfying('Set sound volume', program, supports('Set sound volume')).

verbRequiresAtLeastOneObjectSatisfying('Edit plain text file', program, supports('Edit plain text file')).
verbRequiresAtLeastOneObjectSatisfying('Edit plain text file', file, plainTextFile).

verbRequiresAtLeastOneObjectSatisfying('Turn off', device, computerDevice).
verbRequiresAtLeastOneObjectSatisfying('Login as different user', device, computerDevice).
verbRequiresAtLeastOneObjectSatisfying('Hybernate', device, computerDevice).

verbRequiresAtLeastOneObjectSatisfying('Edit DOC file', file,  ext(doc)).
verbRequiresAtLeastOneObjectSatisfying('Edit DOC file', program, supports('Edit DOC file')).

verbRequiresAtLeastOneObjectSatisfying('Edit SXW file', file,  ext(sxw)).
verbRequiresAtLeastOneObjectSatisfying('Edit SXW file', program, supports('Edit SXW file')).

verbRequiresAtLeastOneObjectSatisfying('Browse the internet', program, supports('Browse the internet')).

verbRequiresAtLeastOneObjectSatisfying('View HTML file', program, supports('View HTML file')).
verbRequiresAtLeastOneObjectSatisfying('View HTML file', file,  htmlFile).

verbRequiresAtLeastOneObjectSatisfying('View DVI file',  file, ext(dvi)).
verbRequiresAtLeastOneObjectSatisfying('View DVI file', program, supports('View DVI file')).

verbRequiresAtLeastOneObjectSatisfying('Create new SXW file', program, supports('Create new SXW file')).


verbRequiresAtLeastOneObjectSatisfying('Edit picture file', program, supports('Edit picture file')).
verbRequiresAtLeastOneObjectSatisfying('Edit picture file', file,  pictureFile).

verbRequiresAtLeastOneObjectSatisfying('Print picture', program, supports('Print picture')).
verbRequiresAtLeastOneObjectSatisfying('Print picture',  file, pictureFile).
verbRequiresAtLeastOneObjectSatisfying('Print picture', device, printerDevice).

verbRequiresAtLeastOneObjectSatisfying('View picture', program, supports('View picture')).
verbRequiresAtLeastOneObjectSatisfying('View picture',  file, pictureFile).

verbRequiresAtLeastOneObjectSatisfying('Change resolution', device, monitorDevice).

verbRequiresAtLeastOneObjectSatisfying('Calculate size', file,  anyFile).

verbRequiresAtLeastOneObjectSatisfying('View print queue', device, printerDevice).
verbRequiresAtLeastOneObjectSatisfying('View print queue', program, supports('View print queue')).

verbRequiresAtLeastOneObjectSatisfying('Restore file from trashcan', program, supports('Restore file from trashcan')).

verbRequiresAtLeastOneObjectSatisfying('Empty trashcan', program, supports('Empty trashcan')).

verbRequiresAtLeastOneObjectSatisfying('Move to trashcan', file,  anyFile).

%anyFile(F):-
%	selected(file, F).
%anyFile( F):-
%	curTable(file, T),
%	t(T, file, F).

anyFile(_).
anyProgram(_).

subtitleFile(F) :-
     ext(sub,F).
subtitleFile(F) :-
     ext(srt,F).

pictureFile(F) :-
	ext(jpg,F).
pictureFile(F) :-
	ext(	png,F).
pictureFile(F) :-
	ext(	 bmp,F).
pictureFile(F) :-
	ext(	 ppm,F).
pictureFile(F) :-
	ext(	 gif,F).

audioFile(F) :-
	ext(	 mp3,F).
audioFile(F) :-
	ext(	 ogg,F).
audioFile(F) :-
	ext(	 ogm,F).
audioFile(F) :-
	ext(	mpc,F).
audioFile(F) :-
	ext(	 flac,F).
audioFile(F) :-
	ext(	 ape,F).
audioFile(F) :-
	ext(	 wma,F).


videoFile(F):-
	 ext(avi, F).
videoFile(F):-
	 ext(mpeg, F).
videoFile(F):-
	 ext(mpg, F).
videoFile(F):-
	 ext(	 asf,F).
videoFile(F):-
	 ext(	 ogg,F).
videoFile(F):-
	 ext(	 ogm,F).
videoFile(F):-
	 ext(	 rm,F).
videoFile(F):-
	 ext(	 wmv,F).

multimediaFile(F) :-
	 ext(	 audioFile,F).
multimediaFile(F) :-
	 ext(	 videoFile,F).

compressedFile(X):-
	ext(	 zip,X).
compressedFile(X):-
	ext(	 rar,X).
compressedFile(X):-
	ext(	 gz,X).
compressedFile(X):-
	ext(	 bz2,X).
compressedFile(X):-
	ext(	 tar,X).
compressedFile(X):-
	ext(	 arj,X).
compressedFile(X):-
	ext(	 ace,X).

readableByHumans(F) :-
	 ext(	 plainTextFile,F).
readableByHumans(F) :-
	 ext(	 pdf,F).
readableByHumans(F) :-
	 ext(	 ps,F).
readableByHumans(F) :-
	 ext(	 eps,F).
readableByHumans(F) :-
	 ext(	 doc,F).
readableByHumans(F) :-
	 ext(	 sxw,F).
readableByHumans(F) :-
	 ext(	 dvi,F).

postscriptFile(F):-
	ext(	 ps,F).
postscriptFile(F):-
	ext(	 eps,F).

plainTextFile(F) :-
	 ext(	 txt,F).
plainTextFile(F) :-
	 ext(	 tex,F).
plainTextFile(F) :-
	 ext(	 srt,F).
plainTextFile(F) :-
	 ext(	 sub,F).
plainTextFile(F) :-
	 ext(	 pl,F).
plainTextFile(F) :-
	 ext(	 cpp,F).
plainTextFile(F) :-
	 ext(	 h,F).
plainTextFile(F) :-
	 ext(	 cc,F).
plainTextFile(F) :-
	 ext(	 bas,F).
plainTextFile(F) :-
	 ext(	 c,F).
plainTextFile(F) :-
	 ext(	 htm,F).
plainTextFile(F) :-
	 ext(	 html,F).
plainTextFile(F) :-
	 ext(	 php,F).

htmlFile(F) :-
	 ext(	 htm,F).
htmlFile(F) :-
	 ext(	 html,F).

cppSourceFile(F) :-
	 ext(	 cc,F).
cppSourceFile(F) :-
	 ext(	 cpp,F).
cppSourceFile(F) :-
	ext(	 h,F).

binaryInstallPackage(F) :-
	ext(	 rpm,F).
binaryInstallPackage(F) :-
	ext(	 deb,F).

% END

supports( 'Take a snapshot of your screen', 'Ksnapshot').

supports( 'Share and download files over the internet', 'Xmule').

supports( 'Read newsgroups', 'Knews').

supports( 'Read newsgroups', 'Pan').

supports( 'View print queue', 'KJobViewer').
supports( 'Set sound volume', 'Kmix').
supports( 'Set sound volume', 'Aumix').
supports( 'Move to trashcan', 'Trashcan').
supports( 'Empty trashcan', 'Trashcan').
supports( 'Restore file from trashcan', 'Trashcan').

supports( 'View DVI file', 'Xdvi').
supports( 'View DVI file', 'Kdvi').


supports( 'View free space on disk', 'Kdiskfree').

supports( 'Partition disk', 'Kparted').
supports( 'Partition disk', 'QTparted').
supports( 'Partition disk', rpmdrake).

supports( 'Burn from ISO', 'K3b').
supports( 'Erase CD-RW', 'K3b').
supports( 'Burn from ISO', 'K3b').
supports( 'Burn data CD', 'K3b').
supports( 'Burn audio CD', 'K3b').
supports( 'Burn video CD', 'K3b').
supports( 'Burn CD from BIN and CUE files', 'K3b').
supports( 'Burn CD from audio and CUE files', 'K3b').

supports( 'Burn from ISO', 'Xcdroast').
supports( 'Burn data CD', 'Xcdroast').

supports( 'Burn video CD', 'Nero For Windows').
supports( 'Burn from ISO', 'Nero For Windows').
supports( 'Burn from ISO', 'Nero For Windows').
supports( 'Burn data CD', 'Nero For Windows').
supports( 'Burn audio CD', 'Nero For Windows').
supports( 'Burn CD from BIN and CUE files', 'Nero For Windows').
supports( 'Burn CD from audio and CUE files', 'Nero For Windows').

supports( 'Play audio', 'Xmms').
supports( 'Enqueue audio for playing', 'Xmms').


supports('Browse the directory of this file', 'Nautilus').



supports( 'Edit picture file', 'The gimp').
supports( 'View picture', 'The gimp').
supports( 'Print picture', 'The gimp').

supports( 'Print picture', 'Kuickshow').
supports( 'View picture', 'Kuickshow').

supports( 'View picture', 'Eye of Gnome').
supports( 'Print picture', 'Eye of Gnome').

supports( 'Play audio', 'Totem').
supports( 'Enqueue audio for playing', 'Totem').
supports( 'Play audio', 'Xine').
supports( 'Enqueue audio for playing', 'Xine').

supports( 'Play movie with subtitle file', 'Mplayer').
supports( 'Play video', 'Mplayer').

supports( 'Play video', 'Xine').
supports( 'Play movie with subtitle file', 'Xine').

supports( 'Play video', 'Totem').

supports( 'Download a file from the internet with resume', 'Kget').
supports( 'Download a file from the internet with resume', 'Opera').
supports( 'Download a file from the internet with resume', 'Downloader for X').

supports( 'Edit plain text file', 'Emacs').
supports( 'Create new text file', 'Emacs').
supports( 'Print text file', 'Emacs').
supports( 'Edit plain text file', 'Vim').
supports( 'Create new text file', 'Vim').
supports( 'Print text file', 'Vim').
supports( 'Edit plain text file', 'Gedit').
supports( 'Print text file', 'Gedit').
supports( 'Create new text file', 'Gedit').
supports( 'Edit plain text file', 'OpenOfficeWriter').
supports( 'Create new SXW file', 'OpenOfficeWriter').
supports( 'Create new PDF file', 'OpenOfficeWriter').
supports( 'Edit SXW file', 'OpenOfficeWriter').
supports( 'Edit DOC file', 'OpenOfficeWriter').
supports( 'View HTML file', 'OpenOfficeWriter').
supports( 'Edit DOC file', 'abiWord').
supports( 'View PDF', 'Xpdf').
supports( 'View PDF', 'Gpdf').
supports( 'View PDF', 'Kghostview').
supports( 'Print PDF', 'Kghostview').

supports( 'Print PDF', 'Kprinter').
supports( 'Print picture', 'Kprinter').


supports( 'Print PDF', 'Xpdf').
supports( 'Print PDF', 'Gpdf').
supports( 'Check new email', 'Kmail').
supports( 'Check new email', 'Mozilla').
supports( 'View HTML file', 'Mozilla').
supports( 'Check new email', 'Evolution').

supports( 'Rip CD', 'Kaudiocreator').
supports( 'Rip CD', 'ripperX').

supports( 'Browse the internet', 'Mozilla').
supports( 'Browse the internet', 'Mozilla Firebird').
supports( 'Browse the internet', 'Konqueror').
supports( 'Browse the internet', 'Opera').

supports( 'View HTML file', 'Konqueror').
supports( 'Send to Konqueror', 'Konqueror').
supports('Browse the directory of this file', 'Konqueror').

supports( 'Extract archive', 'Ark').
supports( 'View content of archive', 'Ark').
supports( 'Extract archive', 'FileRoller').
supports( 'Add to single archive', 'FileRoller').
supports( 'Add to different archives', 'FileRoller').
supports( 'View content of archive', 'FileRoller').
supports( 'View calendar and schedule', 'Korganizer').
supports( 'View calendar and schedule', 'Evolution').

t('All times', time, 'Now').
t('All times', time, '15 seconds from now').
t('All times', time, 'Every 32 minutes starting from now').

t('All devices', device, X):-
	 cdBurningDevice(X).
t('All devices',device, X):-
	 computerDevice(X).
t('All devices', device, X):-
	 ejectableDevice(X).
t('All devices', device, X):-
	 printerDevice(X).
t('All devices', device, X):-
	 keyboardDevice(X).
t('All devices', device, X):-
	 audioCardDevice(X).
t('All devices', device, X):-
	 writableDisk(X).
t('All devices', device, X):-
	 monitorDevice(X).
t('All devices', device, X):-
	 mouseDevice(X).

t('All programs', program, S):-
	supports(_, S).
t('All verbs',verb, V):-
	verbRequiresAtLeastOneObjectSatisfying(V, _, _). 

theItemIsFiled(I, Kind):-
	table(Kind, T, real),
	\+predefinedTable(Kind, _, T),
	t(T, Kind, I).

computerDevice('My Computer').  

printerDevice('My printer').
ejectableDevice('My CD writer').
ejectableDevice('My CD reader').
ejectableDevice('Floppy Drive').
cdBurningDevice('My CD writer').
writableDisk('Floppy Drive').
writableDisk('Primary Hard disk').
keyboardDevice('My Keyboard').
mouseDevice('My Mouse').
audioCardDevice('My sound card').
monitorDevice('My monitor').
cdDevice('My CD writer').
cdDevice('My CD reader').
dvdDevice('My CD reader').

shellName('Xpdf', xpdf).
shellName('Xdvi', xdvi).
shellName('Kdvi', kdvi).
shellName('Kparted', kparted).
shellName('QTparted', qtparted).
shellName('rpmdrake', rpmdrake).
shellName('Kaudiocreator', kaudiocreator).
shellName('ripperX', ripperX).
shellName('Ark', ark).
shellName('FileRoller', 'file-roller').
shellName('The gimp', gimp).
shellName('Korganizer', korganizer).
shellName('Evolution', evolution).
shellName('Kmail', kmail).
shellName('Kghostview', kghostview).
shellName('Gpdf', gpdf).
shellName('Mozilla', mozilla).
shellName('Emacs', emacs).
shellName('Vim', vim).
shellName('Gedit', gedit).
shellName('Totem', totem).
shellName('Xmms', xmms).
shellName('Xine' , xine).
shellName('Mplayer', mplayer).
shellName('Konqueror', konqueror).
shellName('Nautilus', nautilus).
shellName('Kprinter', kprinter).
shellName('Aumix', aumix).
shellName('Kmix', kmix).
shellName('KJobViewer', kjobviewer).
shellName('Kuickshow', kuickshow).
shellName('Eye of Gnome', eog).




pair(X, Y, L):-
	member(X, L),
	member(Y, L),
	X\=Y.

kind(file).
kind(device).
kind(program).
kind(verb).
objectKind(file).
objectKind(device).
objectKind(program).


onVerbSelected(V/*in*/) :-  
	selected(verb, V),
	writeln('>>>ERROR: verb already selected'),
	throw(should-never-happen).
onVerbSelected(V):-
	\+selected(verb, V),
	retractall(selected(verb, _)),
	assert(selected(verb, V)),
	writeln(debug-1),
	updateItemsAfterSelectionChanged(verb),
	writeln(debug-2).

updateRedArrows(VisibleVerbs):-
	VisibleVerbs=[_],
	updateVerbArrow(VisibleVerbs), % of inexplicable gtk bug. if the verb arrow is visible, the go button is not.
	% the problem is not in the prolog code, but in C++. Since with another implementation of MyLabel3 it works.
	widgetHide(arrowImage(program)),
	widgetHide(arrowImage(device)),
	widgetHide(arrowImage(file)),
	forall(missingKind(K, VisibleVerbs), widgetShow(arrowImage(K))),
	(\+selected(time, _)->widgetShow(arrowImage(time));widgetHide(arrowImage(time))).
updateRedArrows(VisibleVerbs):-
	VisibleVerbs\=[_],
	updateVerbArrow(VisibleVerbs),
	widgetHide(arrowImage(program)),
	widgetHide(arrowImage(device)),
	widgetHide(arrowImage(file)),
	widgetHide(arrowImage(time)).
updateVerbArrow(VisibleVerbs):-
	theVerbsAreAllComplete(VisibleVerbs) -> widgetShow(arrowImage(verb)) ; widgetHide(arrowImage(verb)).

theVerbsAreAllComplete(Verbs):-
	forall(member(V, Verbs), theVerbIsComplete(V)).


missingKind(_, VisibleVerbs):-
	VisibleVerbs\=[_].
missingKind(program, VisibleVerbs):-
	VisibleVerbs= [V],
	verbRequiresAtLeastOneObjectSatisfying(V, program, Pred),
	forall(selected(program, P), \+call(Pred,P)).
missingKind(device, VisibleVerbs):-
	VisibleVerbs = [V],
	verbRequiresAtLeastOneObjectSatisfying(V, device, Pred),
	forall(selected(device, D), \+call(Pred, D)).
missingKind(file, VisibleVerbs):-
	VisibleVerbs = [V],
	verbRequiresAtLeastOneObjectSatisfying(V, file, Pred),
	forall(selected(file, F), \+call(Pred, F)).
	

	
	

decideObjectVisibility(O, Kind, VisibleVerbs):-
	theObjectIsVisible(O, Kind, VisibleVerbs),
	checkListBoxSetItemVisibility(checkListBoxOfKind(Kind), O, visible).
decideObjectVisibility(O, Kind, VisibleVerbs):-
	\+theObjectIsVisible(O, Kind, VisibleVerbs),
	checkListBoxSetItemVisibility(checkListBoxOfKind(Kind), O, hidden).
theObjectIsVisible(O,K, _):-
	selected(K, O).
theObjectIsVisible(O, K, VisibleVerbs):-
	\+selected(K, O),
	showHiddenFiles,
	theNameMatchesFilter(K, O),
	member(V, VisibleVerbs), % ... but it is required by some verb
	%  critico se nella def di verbRequiresAtLeastOneObjectSatisfying c' qualcosa che
				% scorre tutti i file...
	verbRequiresAtLeastOneObjectSatisfying(V, K, P),
	call(P, O).
theObjectIsVisible(O, K, VisibleVerbs):-
	nonvar(K),
	\+showHiddenFiles,
	\+selected(K, O),
	theNameMatchesFilter(K, O),
	(K\=file ; \+hiddenFile(O)),
	member(V, VisibleVerbs), % ... but it is required by some verb
	%  critico se nella def di verbRequiresAtLeastOneObjectSatisfying c' qualcosa che
				% scorre tutti i file...
	verbRequiresAtLeastOneObjectSatisfying(V, K, P),
	call(P, O).

theNameMatchesFilter(K, _):-
	entryGetText(containingTextEntry(K), T),
	T=''.
theNameMatchesFilter(K, O):-
	K\=file,
	entryGetText(containingTextEntry(K), T),
	T\='',
	downcase_atom(T, T2),
	downcase_atom(O, O2),
	sub_atom(O2, _, _, _, T2).
theNameMatchesFilter(file, O):-
	entryGetText(containingTextEntry(file), T),
	T\='',
	downcase_atom(T, T2),
	file_base_name(O, BaseName),
	downcase_atom(BaseName, O2),
	sub_atom(O2, _, _, _, T2).
	
onVerbDeselected(VV/*in*/):-
	retract(selected(verb, VV)),
	writeln(onVerbDeselected-1),
	ifItIsNowFilteredAwayRemoveItFromList(verb, VV),
	writeln(onVerbDeselected-2),
	updateItemsAfterSelectionChanged(verb),
	writeln(onVerbDeselected-3).

enableEditVerbFeaturesButtonIfNeeded:-
	selected(verb, _),
	widgetEnable(editFeaturesButton(verb)).
enableEditVerbFeaturesButtonIfNeeded:-
	\+selected(verb, _),
	widgetDisable(editFeaturesButton(verb)).


createActionFromSelection(V, A):-
	findall(F, selected(file, F), Files),
	findall(P, selected(program, P), Programs),
	findall(D, selected(device, D), Devices),
	A=[V, Files, Programs, Devices].


onTimer(Tasks, NewTasks):-
	onTimer([], Tasks, NewTasks).
onTimer(Before, [], Before).
onTimer(Before, [H|T], T2):-
	H=(Time, _),
	\+timeHasPassed(Time),
	onTimer([H|Before]/*keep H*/, T, T2).
onTimer(Before, [H|T], T2):-
	H=(Time, A),
	timeHasPassed(Time),
	executeActionNow(A),
	onTimer(Before/*drop H*/, T, T2).
timeHasPassed(T):-
	get_time(CurrentTime),
	CurrentTime>T.
	
onFilesDeletionConfirmed(Tasks, NewTasks):-
	% REGION choose one time at random @@ we should process all selected times!
	selected(time, T),
	convertTimeToNumber(T, N),
	% END
	createActionFromSelection('Delete', A),
	NewTasks2 = [ ( N, A)| Tasks],
	widgetDestroy(deleteChoiceBox),
	% REGION in case the time was Now, execute the action immediately
	onTimer(NewTasks2, NewTasks).
        % END

convertTimeToNumber('Now', T):-
	get_time(T).
convertTimeToNumber('15 seconds from now', T):-
	get_time(T2),
	T is T2+15.

deleteConfirmationLoop(Tasks, NewTasks):-
	sleepUntilEventIsRaised(E),
	( E = dialogDestroyed(deleteChoiceBox)->
	       (writeln(deletionCanceled), NewTasks = Tasks);
	  E=buttonClicked(yesButton(deleteChoiceBox))->
	       onFilesDeletionConfirmed(Tasks, NewTasks);
	  E=buttonClicked(noButton(deleteChoiceBox))->
        	(widgetDestroy(deleteChoiceBox), NewTasks = Tasks);
	  deleteConfirmationLoop(Tasks, NewTasks)).



renameLoop(F):-
	sleepUntilEventIsRaised(E),
	( E = buttonClicked(renameButton)->
	    onRenameButtonClicked(F);
	  E = dialogDestroyed(renameDialog)->
	    true;
	  renameLoop(F)).
onRenameButtonClicked(F):-
	entryGetText(newNameEntry, NewName),
	NewName = F,
	widgetDestroy(renameDialog).
onRenameButtonClicked(F):-
	entryGetText(newNameEntry, NewName),
	NewName \= F,
	file_directory_name(F, Dir),
	addSlashToDirIfNeeded(Dir, DirSlash),
	concat_atom([DirSlash, NewName], NewNameWithPath),
	concat_atom(['mv \'', F, '\'  \'', NewNameWithPath, '\''], Command),
	shell(Command, Result),
	concat_atom(['Error in the command\n ', Command], ErrorText),
	(Result\=0->
	    messageBox(ErrorText, mainWindow);
	    (% REGION also sets ext/2, maybe improperly
	     removeOldFileFromTablesAndListBoxAndAddNewFileToSameTablesAndListBox(F, NewNameWithPath),
	     % END
	        % REGION if the file has changed extension... @@@ remove it from old table ext/2 and put it into new!
		file_name_extension(_, NewExt, NewName),
		file_name_extension(_, OldExt , F),
		(NewExt=OldExt -> true; 
		    (retractall(ext(_,NewNameWithPath)), % bug fixed: the old file may not
                                                         % have an extension! Therefore the new file may
                                                         % not have an extension either. Therefore retractall.
			assertExtensionIfAny(NewNameWithPath))),
		% END
		forall(selected(file, F),
		       checkListBoxSetButtonCheckState(checkListBoxOfKind(file), F, unchecked)),
		retractall(selected(file, _)),
		updateItemsAfterSelectionChanged(file),
		% REGION log
		get_time(Time),
		convert_time(Time, TimeString),
		concat_atom([TimeString, ': Renamed file: ', F, ' to  ', NewNameWithPath , '.'], Text),
		textViewAddLine(logTextView, Text))),
		% END
	widgetDestroy(renameDialog).

executeActionNow(['Rename', Files, _, _]):-
	Files = [F],
	dialogNew(renameDialog, mainWindow, 'Rename file', 'img/icon.png'),
	widgetShow(renameDialog),

	vBoxNew(renameBox),
	widgetShow(renameBox),
	dialogAddToVBox(renameDialog, renameBox),

	labelNew(renameLabel, 'Please type a new name or modify the existing one:', noLineWrap),
	widgetShow(renameLabel),
	boxPackStart(renameBox, renameLabel, expand, fill),

	entryNew(newNameEntry),
	widgetShow(newNameEntry),
	boxPackStart(renameBox, newNameEntry, expand, fill),
	file_base_name(F, Base),
	entrySetText(newNameEntry, Base),

	buttonNew(renameButton, 'Rename'),
	widgetShow(renameButton),
	boxPackStart(renameBox, renameButton, doNotExpand, doNotFill),

	renameLoop(F).

	    
executeActionNow(['Delete', Files, _Programs, _Devices]):-
        % NOTE: it is not guaranteed that the files
        % exist and are in the t/3, ext/2 and hiddenFile/1 tables!
	forall(member(F, Files), exists_file(F)),
	fromListOfFilesToQuotedString(Files, String),
	concat_atom(['rm -f ', String], Command),
	shell(Command, _),
	% REGION remove the file from all tables where it could be: t/3, ext/2 and hiddenFile/1
	forall(member(F1, Files),
	      forall( (t(T, file, F1), table(file, T, real)), % only non derived tables!
		     retractall(t(T, file, F1)))), % it might already have been removed!
	forall(member(F2, Files),
	       forall(ext(E, F2),
		      retractall(ext(E, F2)))),
	forall(member(F3, Files),
	       retractall(hiddenFile(F3))),
	% END
	% REGION remove the file from the listbox.
	% remove it for real, do not just hide. Otherwise at THE NEXT SORT IT WILL CRASH!
	forall(member(F5, Files),
	       (checkListBoxItemExists(checkListBoxOfKind(file), F5) -> % @@ bug here
	           checkListBoxDeleteItem(checkListBoxOfKind(file), F5) ;
	           true)),
	% END
	forall(member(F4, Files),
	       retractall(selected(file, F4))),
	% REGION redraw list 
	updateItemsAfterSelectionChanged(file),
	% END
	% REGION log
	concat_atom(['Deleted files: ', String], Text),
	addStringToLog(Text).
        % END
executeActionNow(['Delete', Files, _Programs, _Devices]):-
	member(F, Files),
	\+exists_file(F),
	concat_atom(['WARNING: Scheduled deletion discarded because the file ',
		     F, ' does not exist anymore. I suggest reparsing the database.'], Text),
	writeln(Text),
	addStringToLog(Text).
executeActionNow(['Move to...', Files, _, _]):-
	Files = [],
	messageBox('Error. No file selected!', mainWindow). 
executeActionNow(['Move to...', Files, _, _]):-
	Files\=[],
	fileOrDirSelection('Please select the destination directory from the above list, then click OK.', Dir),
	ifTheDirIsValidDoTheMovement(Dir, Files).
	
	
executeActionNow(['Play audio', Files, Programs, _]):-
	sort(Files, SortedFiles),
	forall(member(P, Programs),
	       (shellName(P, P2),
		   fromListOfFilesToQuotedString(SortedFiles, String),
		   concat_atom([P2, ' ', String, '  &'], Command),
		   writeln(Command),
		   shell(Command, _))),
	addStringToLog('Played audio').
executeActionNow(['Play video', Files, Programs, _]):-
	sort(Files, SortedFiles),
	forall(member(P, Programs),
	       (shellName(P, P2),
		   fromListOfFilesToQuotedString(SortedFiles, String),
		   concat_atom([P2, ' ', String, '  &'], Command),
		   writeln(Command),
		   shell(Command, _))),
	addStringToLog('Played video').

ifTheDirIsValidDoTheMovement(Dir, _):-
	\+exists_directory(Dir),
	messageBox('This is not a valid directory', mainWindow).
ifTheDirIsValidDoTheMovement(Dir, _):-
	exists_directory(Dir),
	\+iAmWatchingTheDir(Dir),
	concat_atom(['The move command cannot be executed because segusoLand is not watching the destination directory: ', Dir,
		     '.\nPlease go to the options dialog box and watch the directory, then retry.'],Text),
	messageBox(Text, mainWindow),
	writeln(ifTheDirIsValidDoTheMovement-succeeded).
ifTheDirIsValidDoTheMovement(Dir, Files):-
	exists_directory(Dir),
	iAmWatchingTheDir(Dir),
	fromListOfFilesToQuotedString(Files, String),
	concat_atom(['mv ', String, ' \'', Dir, '\''], Command),
	shell(Command, Result), % 0=ok
	concat_atom(['Error in the command\n ', Command], ErrorText),
	(Result\=0->
	    messageBox(ErrorText, mainWindow);
	    (removeFromOldTablesAndCheckListBoxAndAddToNewTablesAndCheckListBox(Files, Dir),
		forall(selected(file, F),
		       checkListBoxSetButtonCheckState(checkListBoxOfKind(file), F, unchecked)),
		retractall(selected(file, _)),
		updateItemsAfterSelectionChanged(file),
		% REGION log
		get_time(Time),
		convert_time(Time, TimeString),
		concat_atom([TimeString, ': Moved files: ', String, ' to dir ', Dir , '.'], Text),
		textViewAddLine(logTextView, Text))).
		% END

removeFromOldTablesAndCheckListBoxAndAddToNewTablesAndCheckListBox(Files, Dir):-
	forall(member(F, Files),
	       (createNewFileNameFromOldNameAndNewDir(F, Dir, NewF),
		   removeOldFileFromTablesAndListBoxAndAddNewFileToSameTablesAndListBox(F, NewF))).
removeOldFileFromTablesAndListBoxAndAddNewFileToSameTablesAndListBox(Old, New):-
	% REGION t/3
	forall(	(t(T, file, Old),
		    table(file, T, real)),
		(retract(t(T, file, Old)),
		    assert(t(T, file, New)))),
	% END
	% REGION ext/2
	forall(ext(E, Old),
	       (retract(ext(E, Old)),
		   assert(ext(E, New)))),
        % END
	% REGION hiddenFile/1
	(hiddenFile(Old)->
	    (retract(hiddenFile(Old)), assert(hiddenFile(New))) ; true),
        % END

	% REGION CheckListBox
	(checkListBoxItemExists(checkListBoxOfKind(file), Old)->
	    (checkListBoxDeleteItem(checkListBoxOfKind(file), Old),
		addWithProperIcon(file, checkListBoxOfKind(file), New))),
	% END
	retractall(selected(file, Old)).
createNewFileNameFromOldNameAndNewDir(F, Dir, NewName):-
	file_base_name(F, Base),
	addSlashToDirIfNeeded(Dir, DirSlash),
	concat_atom([DirSlash,  Base], NewName).


addOneActionForEachSelectedTime(A, Tasks, NewTasks):-
	selectedTimes(TimeList),
	addOneActionForEachSelectedTime_Aux(A, TimeList, Tasks, NewTasks).
addOneActionForEachSelectedTime_Aux(_A, [], Tasks, Tasks).
addOneActionForEachSelectedTime_Aux(A, [Time|OtherTimes], Tasks, NewTasks):-
	convertTimeToNumber(Time, N),
	NewTasks2 = [ (N, A)|Tasks],
	addOneActionForEachSelectedTime_Aux(A, OtherTimes, NewTasks2, NewTasks).
	
	
onVerbExecuted('Delete', Tasks, NewTasks):-
	findall(F, selected(file, F), Files),
	fromListOfFilesToQuotedString(Files, String),
	concat_atom(['Are you sure you want to delete the following files and all their attributes? \n', String], Text),
	choiceBox(deleteChoiceBox, 'Confirm deletion', Text, 'Yes I am',  'Forget that'), 
	deleteConfirmationLoop(Tasks, NewTasks).
onVerbExecuted('Move to...', N, N):-
	selectedTimes(['Now']),
	createActionFromSelection('Move to...', A),
	executeActionNow(A).
onVerbExecuted('Move to...', N, N):-
	\+selectedTimes(['Now']),
	messageBox('This action cannot be executed in the future. Please select only the time \'Now\' and retry.', mainWindow).
onVerbExecuted('Rename', N, N):-
	\+selectedTimes(['Now']),
	messageBox('This action cannot be executed in the future. Please select only the time \'Now\' and retry.', mainWindow).
onVerbExecuted('Rename', N, N):-
	selectedTimes(['Now']),
	createActionFromSelection('Rename', A),
	executeActionNow(A).
onVerbExecuted('Play audio', Tasks, NewTasks):-
	createActionFromSelection('Play audio', A),
	addOneActionForEachSelectedTime(A, Tasks, NewTasks2),
	addStringToLog('Enqueued action: Play audio'),
	onTimer(NewTasks2, NewTasks). % in case one of the times was Now, executes A at once and pop it out.
onVerbExecuted('Play video', Tasks, NewTasks):-
	createActionFromSelection('Play video', A),
	addOneActionForEachSelectedTime(A, Tasks, NewTasks2),
	addStringToLog('Enqueued action: Play video'),
	onTimer(NewTasks2, NewTasks). % in case one of the times was Now, executes A at once and pop it out.
onVerbExecuted('View free space on disk', N, N):-
	selected(program, 'Kdiskfree'),
	Command = 'kdf &',
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Partition disk', N, N):-
	selected(program, P),
	shellName(P, P2),
	concat_atom([P2, '&'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Rip CD', N, N):-
	selected(program, P),
	shellName(P, P2),
	concat_atom([P2, '&'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Print picture', N, N):-
	selected(program, P),
	shellName(P, P2),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom([P2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Enqueue audio for playing', N, N):-
	selected(program, P),
	shellName(P, P2),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom([P2, ' --enqueue ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Edit SXW file', N, N):-
	selected(program, 'OpenOfficeWriter'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['soffice  ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Edit DOC file', N, N):-
	selected(program, 'OpenOfficeWriter'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['soffice  ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Edit picture file', N, N):-
	selected(program, 'The gimp'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['gimp-remote -n ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('View print queue', N, N):-
	selected(program, 'KJobViewer'),
	shell('kjobviewer &', _).
onVerbExecuted('Burn data CD', N, N):-
	selected(program, 'K3b'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['k3b --data ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(k3b, Status).
onVerbExecuted('Print PDF', N, N):-
	selected(program, 'Kprinter'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['kprinter ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Take a snapshot of your screen', N, N):-
	selected(program, 'Ksnapshot'),
	shell('ksnapshot &', _).
onVerbExecuted('Burn audio CD', N, N):-
	selected(program, 'K3b'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['k3b --audio ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Burn from ISO', N, N):-
	selected(program, 'K3b'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['k3b --isoimage ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Burn video CD', N, N):-
	selected(program, 'K3b'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['k3b --vcd ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Burn CD from BIN and CUE files', N, N):-
	selected(program, 'K3b'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['k3b --binimage ', String, '  &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('View picture', N, N):-
	selected(program, P),
	shellName(P, P2),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom([P2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(P, Status).
onVerbExecuted('View DVI file', N, N):-
	selected(program, P),
	shellName(P, P2),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom([P2, ' ', String, '  &'], Command),
	writeln(Command),
	shell(Command, Status),
	displayProgramNotInstalledIfNecessary(P, Status).
onVerbExecuted('Erase CD-RW', N, N):-
	selected(device, C),
	cdBurningDevice(C),
	shell('k3b --erase &', _).
onVerbExecuted('View calendar and schedule', N, N):-
	selected(program, P),
	shellName(P, P2),
	concat_atom([P2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Set sound volume', N, N):-
	selected(program, P),
	shellName(P, P2),
	concat_atom([P2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Edit plain text file', N, N):-
	selected(program, P),
	shellName(P, P2),
	selected(file, F),
	concat_atom([P2, ' \"',F,  '\" &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Extract archive', N, N):-
	selected(program, 'Ark'),
	selected(file, F),
	concat_atom(['ark --extract \"', F, '\" &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Add to single archive', N, N):-
	selected(program, 'FileRoller'),
	findall(F, selected(file, F), Files),
	sort(Files, Sorted),
	fromListOfFilesToQuotedString(Sorted, String),
	concat_atom(['file-roller --add ', String, '  '], Command), % NOT in background!
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Extract archive', N, N):-
	selected(program, 'FileRoller'),
	writeln('selected FileRoller'),
	selected(file, F),
	concat_atom(['file-roller --extract \"', F, '\" &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('View content of archive', N, N):-
	selected(program, P),
	shellName(P, P2),
	selected(file, F),
	concat_atom([P2, ' \"', F, '\" &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Check new email', N, N):-
	selected(program, P),
	shellName(P, P2),
	concat_atom([P2, ' &'], Command),
	writeln(Command),
	shell(Command, _).
onVerbExecuted('Browse the internet', N, N):-
	selected(program, P),
	shellName(P, P2),
	concat_atom([P2, ' &'], Command),
	shell(Command , _).
onVerbExecuted('Send to Konqueror', N, N):-
	selected(file, F),
	concat_atom(['konqueror \'', F , '\' &'], Command),
	shell(Command, _).
onVerbExecuted('Browse the directory of this file', N, N):-
	findall(X, selected(file, X), Files),
	Files\=[_],
	messageBox('You cannot select more than one file with this verb.', mainWindow).
onVerbExecuted('Browse the directory of this file', N, N):-
	findall(X, selected(file, X), [F]),
	selected(program, P),
	shellName(P, Program),
	file_directory_name(F, D),
	concat_atom([Program, ' \'', D , '\' &'], Command),
	shell(Command, _).
onVerbExecuted('View PDF', N, N):-
	selected(file, F),
	selected(program, P),
	shellName(P, P2),
	concat_atom([P2, ' \'', F , '\' &'], Command),
	shell(Command, _).
onVerbExecuted(V/*in*/, N, N):-
	concat_atom(['Action \"', V, '\" not implemented :-]'], Text),
	messageBox(Text, mainWindow).




fromListOfFilesToQuotedString(List, String):-
	fromListOfFilesToQuotedStringAux(List, '', String).

fromListOfFilesToQuotedStringAux([], CurString, CurString).
fromListOfFilesToQuotedStringAux([H|T], CurString, NewString):-
	H\=[],
	concat_atom([CurString, ' \"', H, '\"'], S2),
	fromListOfFilesToQuotedStringAux(T, S2, NewString).


displayProgramNotInstalledIfNecessary(_, 0).
displayProgramNotInstalledIfNecessary(Program, Status):-
	Status \= 0,
	concat_atom([Program, ' is not installed.'], M),
	messageBox(M, mainWindow).






onObjectSelected(K/*in*/, O/*in*/):-
	selected(K, O).
onObjectSelected(K, O):-
	\+selected(K, O),
	assert(selected(K, O)),
	updateItemsAfterSelectionChanged(K).
	%(selected(time,_)->widgetHide(arrowImage(time)); widgetShow(arrowImage(time))).


updateItemsAfterSelectionChanged(Kind):-
	% REGION narrow verbs
	curTable(verb, TableVerbs),
	forall(t(TableVerbs, verb, V), decideVerbVisibility(V)),
	% END
	% REGION narrow objects to only those objects that are required by a visible verb!
	% bug fixed: setof fails if you are in a a category with no verbs!
	findall(V, theVerbIsVisible(V), VisibleVerbs),
	sort(VisibleVerbs, VisibleVerbsSorted),
	forall(objectKind(K),
	       (curTable(K, T),
		   forall(t(T,K, O), decideObjectVisibility(O,K, VisibleVerbsSorted)))),
	% END
	enableEditFeaturesButtonIfNeeded(Kind),
	setGoButton,
	updateRedArrows(VisibleVerbsSorted).

% the edit button for a kind K is visible if and only if there is exactly one item selected with kind K.
editFeaturesButtonVisible(verb):-
	selected(verb, _).
editFeaturesButtonVisible(Kind):-
	Kind\=verb,
	selected(Kind, M),
	curTable(Kind, Table),
	t(Table, Kind, M),
	forall(selected(Kind, X),
	       (M=X ; (curTable(Kind, T), \+t(T, Kind, X)))).


enableEditFeaturesButtonIfNeeded(time).
enableEditFeaturesButtonIfNeeded(K):-
	K\=time,
	editFeaturesButtonVisible(K),
	widgetEnable(editFeaturesButton(K)),
	(K=file->widgetEnable(dispatchButtonNewbie); true).
enableEditFeaturesButtonIfNeeded(K):-
	K\=time,
	\+editFeaturesButtonVisible(K),
	widgetDisable(editFeaturesButton(K)),
	(K=file->widgetDisable(dispatchButtonNewbie); true).



decideVerbVisibility(V):-
	theVerbIsVisible(V),
	checkListBoxSetItemVisibility(checkListBoxOfKind(verb), V, visible).
decideVerbVisibility(V):-
	\+theVerbIsVisible(V),
	checkListBoxSetItemVisibility(checkListBoxOfKind(verb), V, hidden).


onObjectDeselected(K, O):-
	retract(selected(K, O)),
	ifItIsNowFilteredAwayRemoveItFromList(K, O),
	updateItemsAfterSelectionChanged(K).
	%(selected(time,_)->widgetHide(arrowImage(time)); widgetShow(arrowImage(time))).

ifItIsNowFilteredAwayRemoveItFromList(K, O):-
	curTable(K, T),
	\+t(T,K, O),
	checkListBoxSetItemVisibility(checkListBoxOfKind(K), O, hidden).
ifItIsNowFilteredAwayRemoveItFromList(K, O):-
	curTable(K, T),
	t(T,K, O).


onNewFilterButtonPressedAux(Kind):- % Kind must be file, program, device or verb
	dialogNew(filterDialog, mainWindow, 'Change view', 'img/filter.png'),
	% REGION create boxes
	vBoxNew(filterDialogBox),
	widgetShow(filterDialogBox),
	dialogAddToVBox(filterDialog, filterDialogBox),


	hBoxNew(buttonsHBox),
	widgetShow(buttonsHBox),
	boxPackStart(filterDialogBox, buttonsHBox, doNotFill, fill),

%	hBoxNew(buttonsHBox2),
%	widgetShow(buttonsHBox2),
%	boxPackStart(filterDialogBox, buttonsHBox2, doNotFill, fill),

	hSeparatorNew(filterSep),
	widgetShow(filterSep),
	boxPackStart(filterDialogBox, filterSep, doNotExpand, fill),

	% END
	(Kind=file->  Text2='View all files (slow)' ; concat_atom(['View all ', Kind, 's'], Text2)),
	
	buttonNew(selectAllObjectsButton, Text2),
	widgetShow(selectAllObjectsButton),
	boxPackStart(buttonsHBox, selectAllObjectsButton, expand, doNotFill),

	concat_atom(['View only ', Kind, 's you have not dispatched yet'], Text3),
	buttonNew(selectUnfiledObjectsButton, Text3),
	widgetShow(selectUnfiledObjectsButton),
	boxPackStart(buttonsHBox,selectUnfiledObjectsButton, expand, doNotFill),
	(Kind\=file -> widgetDisable(selectUnfiledObjectsButton); true),

	hBoxNew(filterLabelBox),
	widgetShow(filterLabelBox),
	boxPackStart(filterDialogBox, filterLabelBox, doNotExpand, fill),
	concat_atom(['I want to see only those ', Kind, 's that have been dispatched as: '], Text),
	
	labelNew(dynamicFilterLabel, Text , lineWrap),
	widgetShow(dynamicFilterLabel),
	boxPackStart(filterLabelBox, dynamicFilterLabel, expand, fill),

	% REGION add Apply button
	buttonNew(applyFilterButton, 'Choose this view'),
	widgetShow(applyFilterButton),
	boxPackStart(filterLabelBox, applyFilterButton, expand, doNotFill),
	widgetDisable(applyFilterButton),
	% END
		
	% REGION create and fill the CheckListBox containing the tables
	checkListBoxNew(featureClassesCheckListBox, noRelief, noWhiteBackground),
	widgetShow(featureClassesCheckListBox),
	boxPackStart(filterDialogBox, featureClassesCheckListBox, expand, fill),
	findall(T, (table(Kind, T, real), \+predefinedTable(Kind, _, T)), Tables),
	% @@ bug: contains repetitions:
	sortByNameInsensitive(Tables, SortedTables),
	forall(member(M, SortedTables),
	       checkListBoxAddItem(featureClassesCheckListBox, M, M,'', '', '', 'img/dir.png', buttonInvisible)),
	% END

	% REGION label remember you can choose more than one
	labelNew(labelRemember, 'Remember you can choose more than one item from the above list. For example, "classical"+"music",  "audio"+"book", "horror"+"movie", "legal"+"document".', lineWrap),
	widgetShow(labelRemember),
	boxPackStart(filterDialogBox, labelRemember, doNotExpand, fill),
	% END
	hSeparatorNew(filterSep2),
	widgetShow(filterSep2),
	boxPackStart(filterDialogBox, filterSep2, doNotExpand, fill),

	% REGION deleteSelectedCategoryButton
	buttonNew(deleteSelectedCategoryButton, 'Delete selected categories'),
	widgetShow(deleteSelectedCategoryButton),
	widgetDisable(deleteSelectedCategoryButton),
	boxPackStart(filterDialogBox, deleteSelectedCategoryButton, doNotExpand, fill),
	% END
	windowMaximize(filterDialog),
	widgetShow(filterDialog),
	changeViewLoop([], Kind),
	writeln(changeViewLoop-terminated).

changeViewLoop(SelectedTables, _):-
	var(SelectedTables),
	throw(error-var-SelectedTables).
changeViewLoop(SelectedTables, K):-
	nonvar(SelectedTables),
	sleepUntilEventIsRaised(E),
	( E=checkListBoxItemSelected(featureClassesCheckListBox, I)->
	     changeViewOnTableSelected(I, K, SelectedTables, NewSelectedTables);
	  E=checkListBoxItemDeselected(featureClassesCheckListBox, I)->
	     changeViewOnTableDeselected(I, K, SelectedTables, NewSelectedTables);
	  E=buttonClicked(applyFilterButton)->
	     onChangeViewButtonPressed(K, SelectedTables);% calls widgetDestroy(filterDialog)
	  E = buttonClicked(deleteSelectedCategoryButton)->
	    onDeleteCategoryButtonPressed(K, SelectedTables, NewSelectedTables);
	  E=buttonClicked(selectUnfiledObjectsButton)->
	     onViewOnlyUnfiledItemsButtonPressed(K); % calls widgetDestroy(filterDialog)
	  E=buttonClicked(selectAllObjectsButton)->
	     onViewAllItemsButtonPressed(K); % calls widgetDestroy(filterDialog)
	  E=dialogDestroyed(filterDialog)->
	      true;
	  (writeln(discardedEvent(E)), NewSelectedTables = SelectedTables)),

	% REGION decide if you need to exit the loop
	(E=dialogDestroyed(filterDialog)->
	    writeln(exitingLoop-changeViewLoop);
	 E=buttonClicked(applyFilterButton)->
	    writeln(exitingLoop-changeViewLoop);
         E=buttonClicked(selectUnfiledObjectsButton)->
	    writeln(exitingLoop-changeViewLoop);
	  E=buttonClicked(selectAllObjectsButton)->
	     writeln(exitingLoop-changeViewLoop);
         changeViewLoop(NewSelectedTables, K)).
         % END


onViewOnlyUnfiledItemsButtonPressed(K):-
	predefinedTable(K, toDispatch, Table),
	retractall(curTable(K, _)),
	assert(curTable(K,Table)),
	retractall(recentView(K, Table, Table)),
	asserta(recentView(K, Table, Table)),
	commonShit(K),
	widgetDestroy(filterDialog).
	

onViewAllItemsButtonPressed(K):-
	predefinedTable(K, all, Table),
	retractall(curTable(K, _)),
	assert(curTable(K,Table)),
	retractall(recentView(K, Table, Table)),
	asserta(recentView(K, Table, Table)),
	commonShit(K),
	widgetDestroy(filterDialog).

% this is called when the user selects a table IN THE CHANGE VIEW  DIALOG
changeViewOnTableSelected(T, K, SelectedTables, NewSelectedTables):- % T is a table
	NewSelectedTables  = [T | SelectedTables],
	widgetEnable(applyFilterButton),
	widgetDisable(selectAllObjectsButton),
	widgetDisable(selectUnfiledObjectsButton),
	updateChangeViewSentence(K, NewSelectedTables),
	% REGION update the deleteSelectedCategoryButton
	fromListToReadableSentence(NewSelectedTables, Readable),
	concat_atom(['Delete the categories: ', Readable], Text),
	buttonSetText(deleteSelectedCategoryButton, Text),
	widgetEnable(deleteSelectedCategoryButton).
        % END
changeViewOnTableDeselected(T, K, SelectedTables, NewSelectedTables):-
	remove(T, SelectedTables, NewSelectedTables),
	(NewSelectedTables = []->
	    (widgetDisable(applyFilterButton), widgetEnable(selectAllObjectsButton),
		widgetDisable(deleteSelectedCategoryButton), buttonSetText(deleteSelectedCategoryButton, 'Delete selected categories')); true),
	((NewSelectedTables=[], K=file)->
	    widgetEnable(selectUnfiledObjectsButton);true),
	updateChangeViewSentence(K, NewSelectedTables).

updateChangeViewSentence(K, SelectedTables):-
	fromListToReadableSentence(SelectedTables, Sentence),
	concat_atom(['I want to view only those ', K, 's that have been dispatched as ', Sentence], Text),
	labelSetText(dynamicFilterLabel, Text).


% REGION fromListToReadableSentence/2. Translates [a,b,c] into a more readable 'a and b and c.'
fromListToReadableSentence([], '...').
fromListToReadableSentence(L, S):-
	L\=[],
	fromListToReadableSentenceAux('', L, S).
fromListToReadableSentenceAux(CurrentSentence, [H], New):-
	concat_atom([CurrentSentence, H, '.'], New).
fromListToReadableSentenceAux(CurrentSentence, L, New):-
	L\=[_],
	L=[H|T],
	concat_atom([CurrentSentence, H, ' and '], NewSentence),
	fromListToReadableSentenceAux(NewSentence, T, New).
% END

onChangeViewButtonPressed(K, SelectedTables):-
	member(_, SelectedTables),
	retractall(curTable(K, _)),
	assertNewClauseWithDerivedTable(K, SelectedTables), % writes curTable(K, _).
	commonShit(K),
	widgetDestroy(filterDialog),
	setGoButton.

computeItemsToBeShown(K, T, Items):-
	K\=file,
	findall(I, (selected(K, I) ; t(T, K, I)),  Items).
computeItemsToBeShown(file, T, Items):-
	showHiddenFiles,
	findall(O, (selected(file, O) ; t(T, file, O)) , Items).
computeItemsToBeShown(file, T, Items):-
	\+showHiddenFiles,
	findall(O, (selectedNotHidden(file, O) ; inTableNotHidden(T, file, O)), Items).
selectedNotHidden(file, O):-
	selected(file, O),
	\+hiddenFile(O).
inTableNotHidden(T, file, O):-
	t(T, file, O),
	\+hiddenFile(O).
	

% commonShit/1. Redraws the CheckListBox after changing the view, sorting, or applying a filter.
% Called by onChangeViewButtonPressed, onViewAllItemsButtonPressed, onHiddenFilesVisibilityToggled, etc.
commonShit(K):-
	checkListBoxHideAllItems(checkListBoxOfKind(K)),
	% REGION now show again what must be shown. We must show all _selected_
	% objects of that kind (even not in current table)
	% that are not hidden files, and all objects of that table that are not hidden files.
	curTable(K, T),
	computeItemsToBeShown(K, T, ItemsToBeShown),
	forall(member(M2, ItemsToBeShown), checkListBoxSetItemVisibility(checkListBoxOfKind(K), M2, visible)),
	% END
	% REGION narrow verbs
	curTable(verb, TableVerbs),
	forall(t(TableVerbs,verb, V), decideVerbVisibility(V)),
	% END
	% REGION narrow objects to only those objects that are required by a visible verb!
	findall(V, theVerbIsVisible(V), VisibleVerbs),
	forall(objectKind(K2),
	       (curTable(K2, T2),
		   % @@ we should remove duplicates for programs
		   forall(t(T2,K2, O2), decideObjectVisibility(O2,K2, VisibleVerbs)))),
	% END
	% REGION Set label representing the current view
	curTable(K, T),
	fromClauseHeadToReadableText(t(T, K, _), Text),
	fromKindToCurrentViewLabel(K, Label),
	eventLabelSetText(Label, Text).
	% END
	



fromClauseHeadToReadableText(Head, Text):-
	clause(Head, t(X, _, _)),
	Text = X.
fromClauseHeadToReadableText(Head, Text):-
	clause(Head,  (t(X, _, _), t(Y, _, _))),
	concat_atom(['(', X, ', ', Y, ')'], Text).
fromClauseHeadToReadableText(Head, Text):-
	clause(Head,  (t(X, _, _), t(Y, _, _),t(Z,_,_ ))),
	concat_atom(['(', X, ', ', Y, ', ', Z,')'], Text).
fromClauseHeadToReadableText(Head, Text):-
	clause(Head, true),
	Head=t(Text, _, _).
fromClauseHeadToReadableText(t(Table,_,_), Table):-
	predefinedTable(_, _, Table).
fromClauseHeadToReadableText(_Head, 'error!').



% this does
%   assert(t(DynPredicate, K, X):-t(T1, K, X), t(T2,K, X)).
assertNewClauseWithDerivedTable(K, SelectedTables):-
	SelectedTables = [T1, T2],
	newconst(DynPredicate),
	asserta(t(DynPredicate, K, X):-(t(T1, K, X), t(T2, K, X))),
	assert(curTable(K, DynPredicate)),
	retractall(recentView(K, DynPredicate, [T1,T2])),
	asserta(recentView(K, DynPredicate, [T1,T2])).
assertNewClauseWithDerivedTable(K, SelectedTables):-
	SelectedTables =  [T1, T2, T3],
	newconst(DynPredicate),
	asserta(t(DynPredicate, K, X):-(t(T1, K,X),t(T2, K, X),t(T3, K, X))),
	assert(curTable(K, DynPredicate)),
	retractall(recentView(K, DynPredicate, [T1,T2,T3])),
	asserta(recentView(K, DynPredicate, [T1,T2,T3])).
assertNewClauseWithDerivedTable(K, SelectedTables):-
	SelectedTables =  [T1, T2, T3, T4],
	newconst(DynPredicate),
	asserta(t(DynPredicate, K, X):-(t(T1, K,X),t(T2, K, X),t(T3, K, X), t(T4, K, X))),
	assert(curTable(K, DynPredicate)),
	retractall(recentView(K, DynPredicate, [T1,T2,T3, T4])),
	asserta(recentView(K, DynPredicate, [T1,T2,T3, T4])).
assertNewClauseWithDerivedTable(K, SelectedTables):-
	SelectedTables = [T1],
	newconst(DynPredicate),
	asserta(t(DynPredicate, K, X):-t(T1, K,X)),
	assert(curTable(K, DynPredicate)),
	retractall(recentView(K, DynPredicate, [T1])), % @@ does not work because dynpredicate is different each time!
	asserta(recentView(K, DynPredicate, [T1])).
assertNewClauseWithDerivedTable(_, SelectedTables):-
	writeln('assertNewClauseWithDerivedTable: case not managed'),
	writeln(SelectedTables),
	throw('case not managed'),
	fail.


	
addWithProperIcon(file, CheckListBox, Name):-
	file_base_name(Name, BaseName),
	file_directory_name(Name, Dir),
	file_name_extension(_, Ext, BaseName),
	typeToIcon(Ext, Icon),
	time_file(Name, Date),
	convert_time(Date, ReadableDate),
	size_file(Name, Size),
	SizeInMB is Size /(1024*1024),
	concat_atom([SizeInMB, ' MB'], SizeInMBText),
	checkListBoxAddItem(CheckListBox, Name, BaseName, Dir,SizeInMBText, ReadableDate, Icon, buttonInvisible).
addWithProperIcon(verb, CheckListBox, Name):-
	verbToIcon(Name, Icon),
	checkListBoxAddItem( CheckListBox, Name, Name,'', '', '',  Icon, buttonVisible).
addWithProperIcon(program, CheckListBox, Name):-
	programToIcon(Name, Icon),
	checkListBoxAddItem(CheckListBox, Name, Name,'', '', '',  Icon, buttonInvisible).
addWithProperIcon(device, CheckListBox, Name):-
	deviceToIcon(Name, Icon),
	checkListBoxAddItem(CheckListBox, Name, Name, '', '', '',Icon, buttonInvisible).



onFeatureSelected2(K, Table):-
	% REGION if the object is already in that table, don't do anything.
	% this happens when I set checks via code
	iAmCurrentlyEditingFeaturesOfItem(O),
	t(Table, K, O).
        % END
onFeatureSelected2(K, Table):-
	iAmCurrentlyEditingFeaturesOfItem(O),
	\+t(Table,K, O),
	assert(t(Table, K, O)),
	% REGION the item must be removed from the to-dispatch table IF it is there
	predefinedTable(K, toDispatch, ToDispatch),
	(t(ToDispatch, K, O)->retract(t(ToDispatch, K, O)); true).
	% END
onFeatureDeselected2(K, Table):-
	iAmCurrentlyEditingFeaturesOfItem(O),
	retract(t(Table, K, O)),
	% REGION maybe the item must be inserted again into the to-dispatch table
	predefinedTable(K, toDispatch, ToDispatch),
	(theItemIsFiled(O, K)->true; assert(t(ToDispatch, K, O))).
	


onEditFeaturesButtonPressedAux(Kind):-
	% REGION  pick the first selected item of that kind. There should be only one so there is no problem.
	selected(Kind, O), 
	% END

	% REGION remember the item you are editing, because the callbacks will need it
	          % (they must display only the features which are not currently assigned to it)
	retractall(iAmCurrentlyEditingFeaturesOfItem(_)),
	assert(iAmCurrentlyEditingFeaturesOfItem(O)),
	% END
	concat_atom(['Editing categories of the ', Kind, ' \"', O, '\":'], Title),
	dialogNew(editFeaturesDialog, mainWindow, Title, 'img/edit-info.png'),
	vBoxNew(editFeaturesBox),
	widgetShow(editFeaturesBox),
	dialogAddToVBox(editFeaturesDialog, editFeaturesBox),
	% REGION insert controls
	concat_atom(['Assign the ', Kind, ' \"', O, '\" to the following categories:'], LabelText),
	labelNew(editFeaturesLabel, LabelText, lineWrap),
	widgetShow(editFeaturesLabel),
	boxPackStart(editFeaturesBox, editFeaturesLabel, doNotExpand, fill),
	
	checkListBoxNew(allFeaturesCheckListBox, relief, noWhiteBackground), %, onFeatureSelected2,onFeatureDeselected2, onFeatureExecuted2),
	widgetShow(allFeaturesCheckListBox),
	boxPackStart(editFeaturesBox, allFeaturesCheckListBox, expand, fill),

	buttonNew(addSelectedFeatureButton, 'Add a new category'),
	widgetShow(addSelectedFeatureButton),
	widgetDisable(addSelectedFeatureButton),
	boxPackStart(editFeaturesBox, addSelectedFeatureButton, doNotExpand, fill),

	entryNew(newFeatureTextBox), %, true, true,   onNewFeatureTextBoxTextChanged),
	widgetShow(newFeatureTextBox),
	boxPackStart(editFeaturesBox, newFeatureTextBox, doNotExpand, fill),
        % END

	% REGION fill tables
	findall(T1, (table(Kind, T1, real), \+predefinedTable(Kind, _, T1)), Tables),
	sortByNameInsensitive(Tables, SortedTables),
	forall(member(M, SortedTables),
	       checkListBoxAddItem(allFeaturesCheckListBox, M, M,
				   '', '', '', 'img/edit-info.png',buttonInvisible)),
	% END

	% REGION check tables the item belongs to
	forall(member(T, SortedTables), %(table(Kind, T, real),\+predefinedTable(Kind, _, T)),
	       (t(T,Kind, O)->checkListBoxSetButtonCheckState(allFeaturesCheckListBox, T, checked);
		   true)),   % raises event checkListBoxItemSelected(_)
	% END

	windowMaximize(editFeaturesDialog),
	widgetShow(editFeaturesDialog),
	editFeaturesLoop(Kind).


editFeaturesLoop(K):-
	sleepUntilEventIsRaised(E),
	(
	  E=checkListBoxItemSelected(allFeaturesCheckListBox, I)->
	     onFeatureSelected2(K, I);
	  E=checkListBoxItemDeselected(allFeaturesCheckListBox, I)->
	     onFeatureDeselected2(K, I);
	  E=entryTextChanged(newFeatureTextBox)->
	     onNewFeatureTextBoxTextChanged;
	  E = buttonClicked(addSelectedFeatureButton)->
	     onAddSelectedFeatureButtonPressed(K);
	  E= dialogDestroyed(editFeaturesDialog)->
	    true;
	  writeln(discardedEvent(E))),

	% REGION decide if you need to exit the loop
	(E=dialogDestroyed(editFeaturesDialog)->
	    writeln(exitingLoop);
         editFeaturesLoop(K)).
         % END

onDeleteCategoryButtonPressed(Kind, SelectedCategories, NewSelectedCategories):-
	/*
	% REGION please wait @@@ this dialog does not appear!
	dialogNew(pleaseWaitDialog, mainWindow, 'Please wait', 'img/icon.png'),
	widgetShow(pleaseWaitDialog),
	labelNew(pleaseWaitLabel, 'Please wait while I am updating the database...', lineWrap),
	widgetShow(pleaseWaitLabel),
	dialogAddToVBox(pleaseWaitDialog, pleaseWaitLabel),
	% END
	*/
	forall(member(T, SelectedCategories),
	       (retractall(t(T, Kind, _)),
		   retract(table(Kind, T, real)),
		   checkListBoxDeleteItem(featureClassesCheckListBox, T))),
	NewSelectedCategories = [],
	%REGION some files may now need to be added to the todispatch table
	forall(t('All files', file, File3),
	       (theItemIsFiled(File3, file)->true; assert(t('Files to dispatch', file, File3)))).
	% END



onAddSelectedFeatureButtonPressed(K):-
	entryGetText(newFeatureTextBox, NewTable),
	table(K, NewTable, real),
	messageBox('A view with this name already exists', mainWindow).
onAddSelectedFeatureButtonPressed(Kind):-
	entryGetText(newFeatureTextBox, NewTable),
	\+table(Kind, NewTable, _),
	assert(table(Kind, NewTable, real)),

	% REGION refresh list
	checkListBoxClear(allFeaturesCheckListBox),
	% REGION fill tables
	findall(T1, (table(Kind, T1, real), \+predefinedTable(Kind, _, T1)), Tables),
	sortByNameInsensitive(Tables, SortedTables),
	forall(member(M, SortedTables), checkListBoxAddItem(allFeaturesCheckListBox, M, M,'', '', '', 'img/edit-info.png', buttonInvisible)),
	% END

	% REGION check tables the item belongs to
	iAmCurrentlyEditingFeaturesOfItem(O),
	forall((table(Kind, T, real), \+predefinedTable(Kind, _, T)),
	       t(T,Kind, O)->checkListBoxSetButtonCheckState(allFeaturesCheckListBox, T, checked);
	       true).		% calls onFeatureSelected2
	% END
	% END

onNewFeatureTextBoxTextChanged:-
	entryGetText(newFeatureTextBox, T),
	T='',
	buttonSetText(addSelectedFeatureButton, 'Add'),
	widgetDisable(addSelectedFeatureButton).
onNewFeatureTextBoxTextChanged:-
	entryGetText(newFeatureTextBox, Text),
	Text\='',
	concat_atom(['Add view \"', Text, '\".'], T),
	buttonSetText(addSelectedFeatureButton, T),
	widgetEnable(addSelectedFeatureButton).
	

	
fromKindToLabel(file, labelCurrentFileFilter).
fromKindToLabel(device, labelCurrentDeviceFilter).
fromKindToLabel(program, labelCurrentProgramFilter).
fromKindToLabel(verb, labelCurrentVerbFilter).

messageBox(Text, Parent):-
	dialogNew(messageDialog, Parent, 'Message', 'img/icon.png'),
	vBoxNew(messageBoxBox),
	widgetShow(messageBoxBox),
	dialogAddToVBox(messageDialog, messageBoxBox),
	
	labelNew(messageLabel, Text, lineWrap),
	widgetShow(messageLabel),
	boxPackStart(messageBoxBox, messageLabel, expand, fill),

	buttonNew(messageOkButton, 'Ok'),
	widgetShow(messageOkButton),
	boxPackStart(messageBoxBox, messageOkButton, expand, fill),
	widgetShow(messageDialog),
	writeln(entering-messagebox-loop),
	messageBoxLoop.

messageBoxLoop:-
	sleepUntilEventIsRaised(E),
	( 
	  E = buttonClicked(messageOkButton)->
	    writeln(i-am-calling-widget-destroy), widgetDestroy(messageDialog);
	  E = dialogDestroyed(messageDialog)->
	    writeln(found-dialog-destroyed);
	  writeln(messageBoxLoop-ignoredEvent(E))),

	(E = dialogDestroyed(messageDialog)->
	    writeln(exitingMessageBoxLoop);
	 E = buttonClicked(messageOkButton)->
	    writeln(exitingMessageBoxLoop);
	 messageBoxLoop).


choiceBox(Id, Title, Text, OkText, CancelText):-
	dialogNew(Id, mainWindow, Title, 'img/icon.png'), 
	vBoxNew(choiceBoxBox(Id)),
	widgetShow(choiceBoxBox(Id)),
	dialogAddToVBox(Id, choiceBoxBox(Id)),
	
	labelNew(choiceLabel(Id), Text, lineWrap),
	widgetShow(choiceLabel(Id)),
	boxPackStart(choiceBoxBox(Id), choiceLabel(Id), expand, fill),

	buttonNew(yesButton(Id), OkText),
	widgetShow(yesButton(Id)),
	boxPackStart(choiceBoxBox(Id), yesButton(Id), doNotExpand, fill),

	buttonNew(noButton(Id), CancelText),
	widgetShow(noButton(Id)),
	boxPackStart(choiceBoxBox(Id), noButton(Id), doNotExpand, fill),
	widgetShow(Id).
	

onDestroyMessageBox.
onMessageButtonPressed:-
	widgetDestroy(messageDialog).

onOptionsButtonPressed:-
	dialogNew(optionsDialog, mainWindow, 'Options', 'img/options.png'),

	vBoxNew(dialogBox),
	widgetShow(dialogBox),
	dialogAddToVBox(optionsDialog, dialogBox),

	labelNew(currentDirsLabel, 'This is the list of directories currently watched by segusoLand. You can add and remove dirs.', lineWrap),
	widgetShow(currentDirsLabel),
	boxPackStart(dialogBox, currentDirsLabel, doNotExpand, fill),

	checkListBoxNew(currentDirsCheckListBox, relief, noWhiteBackground),
	widgetShow(currentDirsCheckListBox),
	boxPackStart(dialogBox, currentDirsCheckListBox, expand, fill),

	% REGION fill the currentDirsCheckListBox with the current dirs
	forall(segusoDir(D), checkListBoxAddItem(currentDirsCheckListBox, D, D, '', '', '', 'img/dir.png', buttonInvisible)),
	% END

	hBoxNew(entryAndBrowseBox),
	widgetShow(entryAndBrowseBox),
	boxPackStart(dialogBox, entryAndBrowseBox, doNotExpand, fill),

	labelNew(addNewDirLabel, 'Add a new dir:', noLineWrap),
	widgetShow(addNewDirLabel),
	boxPackStart(entryAndBrowseBox, addNewDirLabel, doNotExpand, fill),
	
	entryNew(newDirTextBox),
	widgetShow(newDirTextBox),
	boxPackStart(entryAndBrowseBox, newDirTextBox, expand, fill),

	imageButtonNew(browseDirButton, 'img/dir.png', 'Browse...'),
	widgetShow(browseDirButton),
	boxPackStart(entryAndBrowseBox, browseDirButton, doNotExpand, fill),

	hBoxNew(addRemoveDirBox),
	widgetShow(addRemoveDirBox),
	boxPackStart(dialogBox, addRemoveDirBox, doNotExpand, fill),
	
	buttonNew(addNewDirButton, 'Add'),
	widgetShow(addNewDirButton),
	boxPackStart(addRemoveDirBox, addNewDirButton, expand, doNotFill),
	widgetDisable(addNewDirButton),

	buttonNew(removeDirButton, 'Remove'),
	widgetShow(removeDirButton),
	boxPackStart(addRemoveDirBox, removeDirButton, expand, doNotFill),
	widgetDisable(removeDirButton),

	windowMaximize(optionsDialog),
	widgetShow(optionsDialog),
	optionDialogLoop([]).

optionDialogLoop(SelectedDirs):-
	var(SelectedDirs),
	throw(error-var-SelectedDirs).
optionDialogLoop(SelectedDirs):-
	nonvar(SelectedDirs),
	sleepUntilEventIsRaised(E),
	(E=checkListBoxItemSelected(currentDirsCheckListBox, I)->
	    onDirSelected(I, SelectedDirs, NewSelectedDirs)
	; E=checkListBoxItemDeselected(currentDirsCheckListBox, I)->
	    onDirDeselected(I, SelectedDirs, NewSelectedDirs)
	; E=entryTextChanged(newDirTextBox)->
	    (onNewDirTextBoxTextChanged, SelectedDirs = NewSelectedDirs)
	; E=buttonClicked(addNewDirButton)->
	    (onAddNewDirButtonPressed, SelectedDirs = NewSelectedDirs);
	E=buttonClicked(removeDirButton)->
	    onRemoveDirButtonPressed(SelectedDirs, NewSelectedDirs);
	E = buttonClicked(browseDirButton)->
	    (onBrowseDirButtonPressed, NewSelectedDirs = SelectedDirs);
	E=dialogDestroyed(optionsDialog)->
	    (NewSelectedDirs = SelectedDirs);
	(writeln(discardedEvent(E)), NewSelectedDirs = SelectedDirs)),
	
	(E=dialogDestroyed(optionsDialog)->
	     writeln(exitingDialogLoop);
	 optionDialogLoop(NewSelectedDirs)).

onBrowseDirButtonPressed:-
	fileOrDirSelection('Please select a directory from the above list, then click OK', Dir),
	entrySetText(newDirTextBox, Dir),
	onNewDirTextBoxTextChanged.

onNewDirTextBoxTextChanged:-
	entryGetText(newDirTextBox, Text),
	expand_file_name(Text, [ExpandedDir]),
	concat_atom(['Add \'', Text, '\''], ButtonText),
	((exists_directory(ExpandedDir), \+iAmWatchingTheDir(Text))->
	    (widgetEnable(addNewDirButton),
		buttonSetText(addNewDirButton, ButtonText));
	    (widgetDisable(addNewDirButton),
		buttonSetText(addNewDirButton, 'Add'))).

onAddNewDirButtonPressed:- 
	entryGetText(newDirTextBox, NewDir),
	% REGION ask whether you want to give all files one or more attributes
	dialogNew(addAttributeToAllFilesInDirDialog, mainWindow, 'Mark the new files?', 'img/icon.png'),

	vBoxNew(addAttributeToAllFilesInDirDialogBox),
	widgetShow(addAttributeToAllFilesInDirDialogBox),
	dialogAddToVBox(addAttributeToAllFilesInDirDialog, addAttributeToAllFilesInDirDialogBox),

	concat_atom(['SegusoLand is now going to import the files contained in ', NewDir, '. You can now choose to mark the files with one or more attributes, in order to reach them quickly from the \"Change view\" dialog. For example, you could mark them with the attributes \"Music\", \"Classical\", \"by Richard Wagner\". If you decide not to do this now, the files will be thrown into the undispatched pool, and will be available from the \"undispatched files\" view, for you to dispatch them later. \n\nWhat do you want to do?'], Text),

	labelNew(testLabel, Text, lineWrap),
	widgetShow(testLabel),
	boxPackStart(addAttributeToAllFilesInDirDialogBox, testLabel, expand, fill),

	buttonNew(markTheFilesButton,
		  'Mark the files now, with one or more attributes \nyou will select in the following dialog box.'),
	widgetShow(markTheFilesButton),
	boxPackStart(addAttributeToAllFilesInDirDialogBox, markTheFilesButton, doNotExpand, fill),
	
	buttonNew(doNotMarkTheFilesButton,
		  'Do not mark the files, but put them in \nthe undispatched files view.'),
	widgetShow(doNotMarkTheFilesButton),
	boxPackStart(addAttributeToAllFilesInDirDialogBox, doNotMarkTheFilesButton, doNotExpand, fill),
	widgetShow(addAttributeToAllFilesInDirDialog),
	% END
	markFileLoop.

markFileLoop:-
	sleepUntilEventIsRaised(E),
	( E = buttonClicked(markTheFilesButton)->
	    onMarkTheFilesNowPressed;
	  E= buttonClicked(doNotMarkTheFilesButton)->
	    onDoNotMarkTheFilesNowPressed;
	  E = dialogDestroyed(addAttributeToAllFilesInDirDialog)->
	    writeln(exitingLoop);
	  markFileLoop).

onMarkTheFilesNowPressed:-
	widgetDestroy(addAttributeToAllFilesInDirDialog), % destroy the previous dialog. No callback is called.
	dialogNew(editFeaturesDialog2, mainWindow, 'Choosing attributes for new files',  'img/edit-info.png'),
	
	vBoxNew(editFeaturesBox),
	widgetShow(editFeaturesBox),
	dialogAddToVBox(editFeaturesDialog2, editFeaturesBox),

	% REGION insert controls
	entryGetText(newDirTextBox, Dir),	
	concat_atom(['Select the attributes to assign to all the files in \"', Dir, ' \":'], LabelText),
	labelNew(editFeaturesLabel, LabelText, noLineWrap),
	widgetShow(editFeaturesLabel),
	boxPackStart(editFeaturesBox, editFeaturesLabel, doNotExpand, fill),
	
	checkListBoxNew(allFeaturesCheckListBox, noRelief, noWhiteBackground),
	widgetShow(allFeaturesCheckListBox),
	boxPackStart(editFeaturesBox, allFeaturesCheckListBox, expand, fill),

	buttonNew(addSelectedFeatureButton, 'Add new view'),%4
	widgetShow(addSelectedFeatureButton),
	widgetDisable(addSelectedFeatureButton),
	boxPackStart(editFeaturesBox, addSelectedFeatureButton, doNotExpand, fill),

	entryNew(newFeatureTextBox), %, true, true,   onNewFeatureTextBoxTextChanged),
	widgetShow(newFeatureTextBox),
	boxPackStart(editFeaturesBox, newFeatureTextBox, doNotExpand, fill),
        % END

	% REGION fill tables
	findall(T1, (table(file, T1, real), \+predefinedTable(file, _, T1)), Tables),
	sortByNameInsensitive(Tables, SortedTables),
	forall(member(M, SortedTables),
	       checkListBoxAddItem(allFeaturesCheckListBox, M, M,
				   '', '', '', 'img/edit-info.png',buttonInvisible)),
	% END
	windowMaximize(editFeaturesDialog2),
	widgetShow(editFeaturesDialog2),
	editFeaturesForNewDirLoop([]).

editFeaturesForNewDirLoop(SelectedTables):-
	var(SelectedTables),
	throw(error-var-SelectedTables).
editFeaturesForNewDirLoop(SelectedTables):-
	nonvar(SelectedTables),
	sleepUntilEventIsRaised(E),
	( E = checkListBoxItemDeselected(allFeaturesCheckListBox, I)->
	      onFeatureDeselectedWhenAddingFeaturesToAllFiles(I, SelectedTables, NewSelectedTables);
	  E = checkListBoxItemSelected(allFeaturesCheckListBox, I)->
	      onFeatureSelectedWhenAddingFeaturesToAllFiles(I, SelectedTables, NewSelectedTables);
	  E = buttonClicked(addSelectedFeatureButton)->
	      (onAddSelectedFeatureButtonPressed2(SelectedTables), NewSelectedTables = SelectedTables);
	  E = entryTextChanged(newFeatureTextBox)->
	     (onNewFeatureTextBoxTextChanged,
              SelectedTables=NewSelectedTables);
	  E = dialogDestroyed(editFeaturesDialog2)->
    	     (onSelectFeatureDialogWhenAddingNewFilesDestroyed(SelectedTables),
	      SelectedTables = NewSelectedTables);
	 (SelectedTables = NewSelectedTables, writeln(ignoredEvent(E)))),

	(E=dialogDestroyed(editFeaturesDialog2)->
	    writeln(exitingLoop);
	 editFeaturesForNewDirLoop(NewSelectedTables)).

onAddSelectedFeatureButtonPressed2(_SelectedTables):-
	entryGetText(newFeatureTextBox, NewTable),
	table(file, NewTable, real),
	messageBox('A view with this name already exists', mainWindow).
onAddSelectedFeatureButtonPressed2(SelectedTables):-
	entryGetText(newFeatureTextBox, NewTable),
	\+table(file, NewTable, _),
	assert(table(file, NewTable, real)),
	% REGION refresh list
	checkListBoxClear(allFeaturesCheckListBox),
	% REGION fill tables
	findall(T1, (table(file, T1, real), \+predefinedTable(file, _, T1)), Tables),
	sortByNameInsensitive(Tables, SortedTables),
	forall(member(M, SortedTables), checkListBoxAddItem(allFeaturesCheckListBox, M, M,'', '', '', 'img/edit-info.png', buttonInvisible)),
	% END
	% REGION recheck any selected tables
	forall(member(T, SelectedTables),
	       checkListBoxSetButtonCheckState(allFeaturesCheckListBox, T, checked)). % calls onFeatureDeselectedWhenAddingFeaturesToAllFiles
	% END
	% END

onFeatureSelectedWhenAddingFeaturesToAllFiles(Feature, SelectedTables, NewSelectedTables):- % features are really tables, hence the name
	NewSelectedTables = [Feature | SelectedTables].
onFeatureDeselectedWhenAddingFeaturesToAllFiles(Feature, SelectedTables, NewSelectedTables):- % features are really tables, hence the name
	remove(Feature, SelectedTables, NewSelectedTables).

onSelectFeatureDialogWhenAddingNewFilesDestroyed([]).
onSelectFeatureDialogWhenAddingNewFilesDestroyed(SelectedTables):-
	SelectedTables\=[],
	% I must now add the files to the given tables.
	entryGetText(newDirTextBox, NewDir),
	assert(segusoDir(NewDir)),
	addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(NewDir),
	% REGION fill the selected tables
	addSlashToDirIfNeeded(NewDir, NewDirSlashed),
	concat_atom([NewDirSlashed, '*'], VisibleFilesQuery),
	concat_atom([NewDirSlashed, '.*'], HiddenFilesQuery),
	expand_file_name(VisibleFilesQuery, VisibleFiles),
	expand_file_name(HiddenFilesQuery, HiddenFiles),
	forall((member(V, VisibleFiles), exists_file(V)/*this is to skip . and ..  and directories! */),
	       forall(member(T, SelectedTables),
		      assert(t(T, file, V)))),
	forall((member(H, HiddenFiles),exists_file(H)),
	       forall(member(T2, SelectedTables),
		      assert(t(T2, file, H)))),
	% END

	% REGION insert the new files into the listbox
	forall((member(V2, VisibleFiles), exists_file(V2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), V2)),
	forall((member(H2, HiddenFiles), exists_file(H2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), H2)),
	% END
	% REGION add the new dir to the list of dirs
	checkListBoxAddItem(currentDirsCheckListBox, NewDir, NewDir, '', '', '', 'img/dir.png', buttonInvisible),
	entrySetText(newDirTextBox, ''),
	widgetDisable(addNewDirButton),
	% END
	% REGION if the files are hidden, hide them
	(showHiddenFiles->true;
	    forall((member(M, HiddenFiles),exists_file(M)),
		   checkListBoxSetItemVisibility(checkListBoxOfKind(file), M, hidden))),
	% END
	
	% REGION if the visible or hidden files  are filtered out, hide them
	% @@
	% END
	
	% REGION if a sorting button is pressed, release it. Necessary because the items have
	% been added at the end, so the sorting is broken
	(imageToggleButtonIsPressed(sortByFileNameButton)->toggleButtonSetPressedState(sortByFileNameButton, released); true),
	(imageToggleButtonIsPressed(sortByFileSizeButton)->toggleButtonSetPressedState(sortByFileSizeButton, released); true),
	(imageToggleButtonIsPressed(sortByFileDateButton)->toggleButtonSetPressedState(sortByFileDateButton, released); true),
	% END

	% REGION remove selection
	forall(selected(file, F),
	       checkListBoxSetButtonCheckState(checkListBoxOfKind(file), F, unchecked)),
	retractall(selected(file, _)),
	updateItemsAfterSelectionChanged(file).
        % END
	
	
onDoNotMarkTheFilesNowPressed:-
	entryGetText(newDirTextBox, NewDir),
	assert(segusoDir(NewDir)),
	addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(NewDir),
	% REGION fill the Files-to-dispatch table
	addSlashToDirIfNeeded(NewDir, NewDirSlashed),
	concat_atom([NewDirSlashed, '*'], VisibleFilesQuery),
	concat_atom([NewDirSlashed, '.*'], HiddenFilesQuery),
	expand_file_name(VisibleFilesQuery, VisibleFiles),
	expand_file_name(HiddenFilesQuery, HiddenFiles),
	forall((member(V, VisibleFiles), exists_file(V)),
	       assert(t('Files to dispatch', file, V))),
	forall((member(H, HiddenFiles),exists_file(H)),
	       assert(t('Files to dispatch', file, H))),
	% END

	% REGION insert the new items into the listbox
	forall((member(V2, VisibleFiles), exists_file(V2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), V2)),
	forall((member(H2, HiddenFiles), exists_file(H2)),
	       addWithProperIcon(file, checkListBoxOfKind(file), H2)),
	% END
	widgetDestroy(addAttributeToAllFilesInDirDialog),% no callback called
	% REGION add the new dir to the list 
	checkListBoxAddItem(currentDirsCheckListBox, NewDir, NewDir, '', '', '', 'img/dir.png', buttonInvisible),
	entrySetText(newDirTextBox, ''),
	widgetDisable(addNewDirButton),
	% END
	% REGION if the items are hidden, hide them
	(showHiddenFiles->true;
	    forall((member(M, HiddenFiles), exists_file(M)),
		   checkListBoxSetItemVisibility(checkListBoxOfKind(file), M, hidden))),
	% END
	% REGION if the visible or hidden files  are filtered out, hide them
	% @@
	% END

	% REGION if a sorting button is pressed, release it. Necessary because the items have
	% been added at the end, so the sorting is broken
	(imageToggleButtonIsPressed(sortByFileNameButton)->toggleButtonSetPressedState(sortByFileNameButton, released); true),
	(imageToggleButtonIsPressed(sortByFileSizeButton)->toggleButtonSetPressedState(sortByFileSizeButton, released); true),
	(imageToggleButtonIsPressed(sortByFileDateButton)->toggleButtonSetPressedState(sortByFileDateButton, released); true),
	% END
	retractall(selected(file, _)).
		      

onDirSelected(D, SelectedDirs, NewSelectedDirs):-
	NewSelectedDirs = [D|SelectedDirs],
	widgetEnable(removeDirButton).

onDirDeselected(D, SelectedDirs, NewSelectedDirs):-
	remove(D, SelectedDirs, NewSelectedDirs),
	(NewSelectedDirs=[] -> widgetDisable(removeDirButton); true).

onRemoveDirButtonPressed(SelectedDirs, NewSelectedDirs):-
	% REGION show confirmation dialog
	   % @@
	% END
	% REGION remove all files in those dirs dir from any table!
	findall(F, (member(D2, SelectedDirs), fileInDir(F, D2)), FilesToRemove),
	sort(FilesToRemove, FilesToRemoveWithoutRepetitions),
	removeFilesFromCheckListBoxAndDatabase(FilesToRemoveWithoutRepetitions), % also redraws the list
	% END
	forall(member(D3, SelectedDirs), retract(segusoDir(D3))),
	% REGION update the currentDirsCheckListBox with the current dirs
	checkListBoxClear(currentDirsCheckListBox),
	forall(segusoDir(D), checkListBoxAddItem(currentDirsCheckListBox, D, D, '', '', '', 'img/dir.png', buttonInvisible)),
	% END
	NewSelectedDirs = [],
	widgetDisable(removeDirButton).


fileInDir(F, D):- 
	t('All files',file, F),
	file_directory_name(F, D1),
	concat_atom([D1, '/'], D2),
	expand_file_name(D, [DExp]),
	(D=D2; D=D1; DExp = D2; DExp = D1).
removeFilesFromCheckListBoxAndDatabase(F):- % F is a list of files with full path name
	% REGION remove the file from all tables where it could be. BUT NOT DERIVED TABLES.
	forall(member(F1, F), 
	       forall( (t(T, file, F1),
			   table(file, T, real)), % ONLY IF IT IS NOT A DERIVED TABLE... i.e. it is not
		      % of the kind t(p1, file, X):- t(computer, file, X), t(book, file, X).
		       retract(t(T, file, F1)))), % BUG FIXED! if t is a derived table, retract fails.
	forall(member(F2, F),
	       forall(ext(E, F2),
		      retract(ext(E, F2)))),
	forall(member(F3, F),
	       (hiddenFile(F3)->retract(hiddenFile(F3)); true)),
	% END
	% REGION remove the file from the listbox.
	% remove it for real, do not just hide. Otherwise at THE NEXT SORT IT WILL CRASH!
	forall(member(F5, F),
	       checkListBoxDeleteItem(checkListBoxOfKind(file), F5)),
	% END
	% REGION redraw list
	retractall(selected(file, _)),
	updateItemsAfterSelectionChanged(file). % reads selected(file, X)
	% END

onDeselectAllButtonPressed:-
	forall( (selected(K, Thing), K\=time),
	       (checkListBoxSetButtonCheckState(checkListBoxOfKind(K), Thing, unchecked),
		   retract(selected(K, Thing)))),
	updateItemsAfterSelectionChanged(file),
	widgetDisable(editFeaturesButton(verb)),
	widgetDisable(editFeaturesButton(device)),
	widgetDisable(editFeaturesButton(program)).
	
	
% main procedure, called by file.cc.
run(DirWhereTheSubdirectory_Img_isLocated):-
	% REGION set the work dir  because it is boring to concat this dir each time I need to read an image. This way
	%  I simply say img/file.png
	working_directory(_, DirWhereTheSubdirectory_Img_isLocated),
	% END

	% REGION splash screen
	windowNew(splashScreen, 'segusoLand is starting up...', 'img/icon.png'),
	labelNew(splashScreenMessage, 'Please wait. segusoLand is starting up...', lineWrap),
	widgetShow(splashScreenMessage),
	containerAdd(splashScreen, splashScreenMessage),
	widgetShow(splashScreen),
	% END

	windowNew(mainWindow, 'segusoLand - by Maurizio Colucci', 'img/icon.png'),
	vBoxNew(mainBoxVertical),
	containerAdd(mainWindow, mainBoxVertical),
	widgetShow(mainBoxVertical),

	% REGION menu
	menuBarNew(menuBar),
	widgetShow(menuBar),
	boxPackStart(mainBoxVertical, menuBar, doNotExpand, fill),

	menuNew(fileMenu),
	widgetShow(fileMenu),
	menuBarAddMenu(menuBar, fileMenu, '_File'),
	menuAddItem(fileMenu, quitItem, '_Quit'),

	menuNew(viewMenu),
	widgetShow(viewMenu),
	menuBarAddMenu(menuBar, viewMenu, '_View'),
	menuAddCheckItem(viewMenu, logItem, '_Log'),
	menuAddCheckItem(viewMenu, advancedItem, '_Advanced interface'),
	menuAddItem(viewMenu, tasksItem, '_Scheduled tasks'),
	menuAddCheckItem(viewMenu, tooltipsItem, '_Tooltips'),



	menuNew(optionsMenu),
	widgetShow(optionsMenu),
	menuBarAddMenu(menuBar, optionsMenu, '_Options'),

	menuAddItem(optionsMenu, directoriesItem, '_Directories'),
	
	menuNew(helpMenu),
	widgetShow(helpMenu),
	menuBarAddMenu(menuBar, helpMenu, 'Help'),

	menuAddItem(helpMenu, aboutItem, '_About the authors'),
	
	

	% END

	frameNew(toolbarFrame),
	widgetShow(toolbarFrame),

	boxPackStart(mainBoxVertical, toolbarFrame, doNotExpand, fill),
	
	hBoxNew(fileSortBox),
	widgetShow(fileSortBox),
	containerAdd(toolbarFrame, fileSortBox),

	imageButtonNew(deselectAllButton, 'img/deselect-all.png', 'Deselect all'),
	buttonSetReliefNone(deselectAllButton),
	widgetSetTooltip(deselectAllButton, 'Deselect all files, verbs, devices and programs (not times).'),
	widgetShow(deselectAllButton),
	boxPackEnd(fileSortBox, deselectAllButton, doNotExpand, doNotFill),

	imageButtonNew(reparseDiskButton, 'img/refresh.png', ''),
	buttonSetReliefNone(reparseDiskButton),
	widgetSetTooltip(reparseDiskButton, 'Synchronizes segusoLand\'s internal database with the disk. You should click this if you created, deleted or moved some file with an external program. Newly created files will appear in the to-dispatch view and in the all-files view.'),
	widgetShow(reparseDiskButton),
	boxPackEnd(fileSortBox, reparseDiskButton, doNotExpand, doNotFill),



	imageToggleButtonNew(showQuickSearchButton, 'img/mbuto.png', ''),
	buttonSetReliefNone(showQuickSearchButton),
	widgetSetTooltip(showQuickSearchButton, 'If you click here, quick-search boxes will appear. Quick search-boxes are text entries where you can type a search string in order to restrict the visible files, verbs, etc.'),
	widgetShow(showQuickSearchButton),
	boxPackEnd(fileSortBox, showQuickSearchButton, doNotExpand, doNotFill),

	vSeparatorNew(refreshSeparator),
	widgetShow(refreshSeparator),
	boxPackStart(fileSortBox, refreshSeparator, doNotExpand, fill),

	% REGION newbie stuff
	
	imageButtonNew(recentViewsButtonNewbie, 'img/1.png', 'Recent views'),
	buttonSetReliefNone(recentViewsButtonNewbie),
	widgetShow(recentViewsButtonNewbie),
	widgetSetTooltip(recentViewsButtonNewbie, 'Quickly get back to a previous view of files'),
	boxPackStart(fileSortBox, recentViewsButtonNewbie, doNotExpand, fill),


	imageButtonNew(changeViewButtonNewbie, 'img/filter.png', 'Change view'),
	buttonSetReliefNone(changeViewButtonNewbie),
	widgetShow(changeViewButtonNewbie),
	widgetSetTooltip(changeViewButtonNewbie, 'Click here to set which files must be visible and which must be hidden. Very useful if you see too many files. This is the equivalent of changing directory in traditional systems.'),
	boxPackStart(fileSortBox, changeViewButtonNewbie, doNotExpand, fill),

	imageButtonNew(dispatchButtonNewbie, 'img/edit-info.png', 'Dispatch file'),
	buttonSetReliefNone(dispatchButtonNewbie),
	widgetShow(dispatchButtonNewbie),
	widgetSetTooltip(dispatchButtonNewbie, 'Click here to catalog the selected file. This is the equivalent of moving the file to a directory in traditional systems.'),
	boxPackStart(fileSortBox, dispatchButtonNewbie, doNotExpand, fill),
	widgetDisable(dispatchButtonNewbie),


        % END

	vSeparatorNew(sortSeparator),
	widgetShow(sortSeparator),
	boxPackStart(fileSortBox, sortSeparator, doNotExpand, fill),
	
	labelNew(sortLabel, 'Sort by', noLineWrap),
	widgetShow(sortLabel),
	boxPackStart(fileSortBox, sortLabel, doNotExpand, fill),
	
	imageToggleButtonNew(sortByFileNameButton, 'img/name.png', ''),
	buttonSetReliefNone(sortByFileNameButton),
	widgetShow(sortByFileNameButton),
	widgetSetTooltip(sortByFileNameButton, 'Sort by file name'),
	boxPackStart(fileSortBox, sortByFileNameButton, doNotExpand, fill),
	
	imageToggleButtonNew(sortByFileSizeButton,'img/size.png', ''),
	buttonSetReliefNone(sortByFileSizeButton),
	widgetSetTooltip(sortByFileSizeButton, 'Sort by file size (biggest file on top)'),
	widgetShow(sortByFileSizeButton),
	boxPackStart(fileSortBox, sortByFileSizeButton, doNotExpand, fill),
	
	imageToggleButtonNew(sortByFileDateButton, 'img/1.png', ''),
	buttonSetReliefNone(sortByFileDateButton),
	widgetShow(sortByFileDateButton),
	widgetSetTooltip(sortByFileDateButton, 'Sort by file date (most recent file on top)'),
	boxPackStart(fileSortBox, sortByFileDateButton, doNotExpand, fill),


	vSeparatorNew(showSeparator),
	widgetShow(showSeparator),
	boxPackStart(fileSortBox, showSeparator, doNotExpand, fill),
	
	labelNew(fileDateDirSizeLabel, 'Show', noLineWrap),
	widgetShow(fileDateDirSizeLabel),
	boxPackStart(fileSortBox, fileDateDirSizeLabel, doNotExpand, fill),
	
	imageToggleButtonNew(showFileDirButton, 'img/dir.png', ''),
	buttonSetReliefNone(showFileDirButton),
	widgetSetTooltip(showFileDirButton, 'Show directories'),
	widgetShow(showFileDirButton),
	boxPackStart(fileSortBox, showFileDirButton, doNotExpand, fill),
	
	imageToggleButtonNew(showFileSizeButton,'img/size.png', ''),
	buttonSetReliefNone(showFileSizeButton),
	widgetSetTooltip(showFileSizeButton, 'Show file sizes'),
	widgetShow(showFileSizeButton),
	boxPackStart(fileSortBox, showFileSizeButton, doNotExpand, fill),
	
	imageToggleButtonNew(showFileDateButton, 'img/1.png', ''),
	buttonSetReliefNone(showFileDateButton),
	widgetShow(showFileDateButton),
	widgetSetTooltip(showFileDateButton, 'Show file dates'),
	boxPackStart(fileSortBox, showFileDateButton, doNotExpand, fill),
	
	imageToggleButtonNew(showHiddenFilesButton, 'img/hidden.png', ''),
	buttonSetReliefNone(showHiddenFilesButton),
	widgetSetTooltip(showHiddenFilesButton, 'Show hidden files'),
	widgetShow(showHiddenFilesButton),
	boxPackStart(fileSortBox, showHiddenFilesButton, doNotExpand, fill),



	%hSeparatorNew(mainSep),
	%widgetShow(mainSep),
	%boxPackStart(mainBoxVertical, mainSep, doNotExpand, fill),
	
	% END

	hBoxNew(mainBox),
	widgetShow(mainBox),
	containerAdd(mainBoxVertical, mainBox),


	vBoxNew(verbBox),
	widgetShow(verbBox),
	boxPackStart(mainBox, verbBox, expand, fill),
	
	%vSeparatorNew(verbSep),
	%boxPackStart(mainBox, verbSep, doNotExpand, fill),
	
	vBoxNew(fileBox),
	widgetShow(fileBox),
	boxPackStart(mainBox, fileBox,  expand, fill),

	%vSeparatorNew(fileSep),
	%boxPackStart(mainBox, fileSep, doNotExpand, fill),

	vBoxNew(programBox),
	widgetShow(programBox),
	boxPackStart(mainBox, programBox, expand, fill),

	%vSeparatorNew(deviceSep),
	%boxPackStart(mainBox, deviceSep, doNotExpand, fill),

	vBoxNew(deviceBox),
	widgetShow(deviceBox),
	boxPackStart(mainBox, deviceBox,  expand, fill),
	

	
	
	

	


	% REGION fill verbs
	writeln(verbs),

	% REGION substring box
	hBoxNew(filterHBox(verb)),
	boxPackStart(verbBox, filterHBox(verb), doNotExpand, fill),

	
	imageNew(mbuto(verb), 'img/mbuto.png'),
	widgetShow(mbuto(verb)),
	boxPackStart(filterHBox(verb), mbuto(verb), doNotExpand, fill),

	imageButtonNew(deleteSubstringFilterButton(verb), 'img/delete-mbuto.png', ''),
	boxPackStart(filterHBox(verb), deleteSubstringFilterButton(verb), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(verb), 'Remove the current substring filter'),
	
	entryNew(containingTextEntry(verb)),
	widgetShow(containingTextEntry(verb)),
	boxPackStart(filterHBox(verb), containingTextEntry(verb), expand, fill),
	widgetSetTooltip(containingTextEntry(verb), 'SUBSTRING SEARCH: If you enter some text here, only verbs whose name contains that text will be shown. Beware: the visible verbs determine the visible objects, devices, and programs.'),

	imageNew(horizArrow(verb), 'img/horiz-arrow.gif'),
	boxPackStart(filterHBox(verb), horizArrow(verb), doNotExpand, fill),


	% END substring box

	frameNew(verbFrame),
	boxPackStart(verbBox, verbFrame, doNotExpand, fill),
	
	hBoxNew(verbFilterBox),
	widgetShow(verbFilterBox),
	containerAdd(verbFrame, verbFilterBox),


	imageButtonNew(editFeaturesButton(verb), 'img/edit-info.png', ''),
	buttonSetReliefNone(editFeaturesButton(verb)),
	widgetShow(editFeaturesButton(verb)),
	boxPackStart(verbFilterBox, editFeaturesButton(verb), expand, fill),
	widgetSetTooltip(editFeaturesButton(verb), 'DISPATCH VERB: Click here to catalog the selected verb'),
	widgetDisable(editFeaturesButton(verb)),

	imageButtonNew(newFilterButton(verb), 'img/filter.png', ''),
	buttonSetReliefNone(newFilterButton(verb)),
	widgetShow(newFilterButton(verb)),
	widgetSetTooltip(newFilterButton(verb), 'CHANGE VERB VIEW: Click here to set which verbs must be visible and which must be hidden. Remember that the visible verbs determine the visible files, devices and programs! If no verb is visible, then nothing is visible.'),
	boxPackStart(verbFilterBox, newFilterButton(verb), expand, fill),

	imageButtonNew(recentFiltersButton(verb), 'img/1.png', ''),
	buttonSetReliefNone(recentFiltersButton(verb)),
	%widgetShow(recentFiltersButton(verb)),
	widgetSetTooltip(recentFiltersButton(verb), 'RECENT VIEWS: Quickly get back to a previous view of verbs.'),
	boxPackStart(verbFilterBox, recentFiltersButton(verb), expand, fill),

	hBoxNew(labelAndArrowBox(verb)),
	widgetShow(labelAndArrowBox(verb)),
	boxPackStart(verbBox, labelAndArrowBox(verb), doNotExpand, fill),

	eventLabelNew(labelCurrentVerbFilter, 'All verbs', lineWrap),
	widgetSetTooltip(labelCurrentVerbFilter, 'This label reminds you which verbs are currently visible. You can change this with the Change-view button.'),
	widgetShow(labelCurrentVerbFilter),
	boxPackStart(labelAndArrowBox(verb), labelCurrentVerbFilter, expand, fill),

	imageNew(arrowImage(verb), 'img/redarrow.gif'),
	boxPackStart(labelAndArrowBox(verb), arrowImage(verb), doNotExpand, fill),

	% REGION Add a fake box because the image is too rightwise
	vBoxNew(youCanClickGo),
	widgetShow(youCanClickGo),
	widgetSetSizeRequest(youCanClickGo, 20, 1),
	boxPackStart(labelAndArrowBox(verb), youCanClickGo, doNotExpand, fill),
	% END
	
	checkListBoxNew(checkListBoxOfKind(verb),noRelief, noWhiteBackground),
	widgetShow(checkListBoxOfKind(verb)),
	boxPackStart(verbBox, checkListBoxOfKind(verb), expand, fill),

	curTable(verb, TableVerbs),
	findall(V, t(TableVerbs,verb, V), Verbs),
	sortByNameInsensitive(Verbs, Verbs2),
	forall(member(X, Verbs2),
	       addWithProperIcon(verb, checkListBoxOfKind(verb), X)),

	% REGION verb sort box
	hBoxNew(verbSortBox),
	%widgetShow(verbSortBox),
	boxPackStart(verbBox, verbSortBox, doNotExpand, fill),
	
	labelNew(verbSortLabel, 'Sort by ', noLineWrap),
	widgetShow(verbSortLabel),
	boxPackStart(verbSortBox, verbSortLabel, doNotExpand, fill),
	
	imageButtonNew(sortByVerbNameButton, 'img/name.png', ''),
	widgetShow(sortByVerbNameButton),
	boxPackStart(verbSortBox, sortByVerbNameButton, expand, fill),
	widgetDisable(sortByVerbNameButton),

	imageButtonNew(sortByVerbUsageButton, 'img/1.png', ''),
	widgetShow(sortByVerbUsageButton),
	boxPackStart(verbSortBox, sortByVerbUsageButton, expand, fill),
	widgetDisable(sortByVerbUsageButton),

	imageButtonNew(sortByVerbRelevanceButton, 'img/size.png', ''),
	widgetShow(sortByVerbRelevanceButton),
	boxPackStart(verbSortBox, sortByVerbRelevanceButton, expand, fill),
	widgetDisable(sortByVerbRelevanceButton),
	% END

	
        % END
	% REGION file panel
	writeln(file-panel),
	% REGION substring box
	hBoxNew(filterHBox(file)),
	%widgetShow(filterHBox(file)),
	boxPackStart(fileBox, filterHBox(file), doNotExpand, fill),


	imageNew(mbuto(file), 'img/mbuto.png'),
	widgetShow(mbuto(file)),
	boxPackStart(filterHBox(file), mbuto(file), doNotExpand, fill),

	imageButtonNew(deleteSubstringFilterButton(file), 'img/delete-mbuto.png', ''),
	boxPackStart(filterHBox(file), deleteSubstringFilterButton(file), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(file), 'Remove the current substring filter'),

	entryNew(containingTextEntry(file)),
	widgetShow(containingTextEntry(file)),
	boxPackStart(filterHBox(file), containingTextEntry(file), expand, fill),
	widgetSetTooltip(containingTextEntry(file), 'SUBSTRING SEARCH: If you enter some text here, only files whose name contains that text will be shown.'),

	imageNew(horizArrow(file), 'img/horiz-arrow.gif'),
	boxPackStart(filterHBox(file), horizArrow(file), doNotExpand, fill),


	% END substring box

	% REGION expert file filter box

	frameNew(fileFrame),
	boxPackStart(fileBox, fileFrame, doNotExpand, fill),

	hBoxNew(filterBox),
	widgetShow(filterBox),
	containerAdd(fileFrame, filterBox),


	imageButtonNew(editFeaturesButton(file), 'img/edit-info.png', ''),
	buttonSetReliefNone(editFeaturesButton(file)),
	widgetShow(editFeaturesButton(file)),
	widgetSetTooltip(editFeaturesButton(file), 'DISPATCH FILE: Click here to catalog the selected file. This is the equivalent of moving the file to a directory in traditional systems.'),
	boxPackStart(filterBox, editFeaturesButton(file), expand, fill),
	widgetDisable(editFeaturesButton(file)),

	imageButtonNew(newFilterButton(file), 'img/filter.png', ''),
	buttonSetReliefNone(newFilterButton(file)),
	widgetShow(newFilterButton(file)),
	widgetSetTooltip(newFilterButton(file), 'CHANGE FILE VIEW: Click here to set which files must be visible and which must be hidden. Very useful if you see too many files. This is the equivalent of changing directory in traditional systems.'),
	boxPackStart(filterBox, newFilterButton(file), expand, fill),

	imageButtonNew(recentFiltersButton(file), 'img/1.png', ''),
	buttonSetReliefNone(recentFiltersButton(file)),
	%widgetShow(recentFiltersButton(file)),
	widgetSetTooltip(recentFiltersButton(file), 'RECENT VIEWS: Quickly get back to a previous view of files'),
	boxPackStart(filterBox, recentFiltersButton(file), expand, fill),
	% END expert file filter box
	
	hBoxNew(labelAndArrowBox(file)),
	widgetShow(labelAndArrowBox(file)),
	boxPackStart(fileBox, labelAndArrowBox(file), doNotExpand, fill),

	imageNew(arrowImage(file), 'img/redarrow.gif'),
	boxPackStart(labelAndArrowBox(file), arrowImage(file), doNotExpand, fill),
	
	eventLabelNew(labelCurrentFileFilter, 'All files', lineWrap),
	widgetSetTooltip(labelCurrentFileFilter, 'This label reminds which files are currently visible. You can change this with the Change-view button. Remember that selecting verbs, devices and programs can hide these files anyway! So this label is only accurate when nothing is selected.'),
	widgetShow(labelCurrentFileFilter),
	boxPackStart(labelAndArrowBox(file), labelCurrentFileFilter, expand, fill),


	checkListBoxNew(checkListBoxOfKind(file), relief, whiteBackground),
	widgetShow(checkListBoxOfKind(file)),
	boxPackStart(fileBox, checkListBoxOfKind(file), expand, fill),

	% REGION read database
	writeln(reading-database),
	createDbIfItDoesNotExist,
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/segusoLand-database'], DatabaseFileNamewithFullPath),
	open(DatabaseFileNamewithFullPath, read, Dat),
	readLoop(Dat), % writes t(Table, Kind, File) and segusoDir(D)
	close(Dat),
	writeln(finished-reading-database),
	% END
	writeln(filling-predefined-tables),
	% REGION load files from disk and put them into the predefined tables
	forall(segusoDir(D), addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(D)),
	% END
	% REGION fill the Files-to-dispatch table
	forall(t('All files', file, File3),
	       (theItemIsFiled(File3, file)->true; assert(t('Files to dispatch', file, File3)))),
	% END
	writeln(filling-predefined-tables-end),
	writeln(filling-checklistbox),
	% REGION fill the CheckListBox with all the files.
	forall(t('All files', file, File),
	       addWithProperIcon(file, checkListBoxOfKind(file), File)),
	% END
	% REGION hide hidden files by default
	forall(hiddenFile(HF), checkListBoxSetItemVisibility(checkListBoxOfKind(file), HF, hidden)),
	% END
	writeln(filling-checklistbox-end),
	


	/*
	% REGION file sort box
	hBoxNew(fileSortBox),
	widgetShow(fileSortBox),
	boxPackStart(fileBox, fileSortBox, doNotExpand, fill),
	
	labelNew(sortLabel, 'Sort by ', noLineWrap),
	widgetShow(sortLabel),
	boxPackStart(fileSortBox, sortLabel, doNotExpand, fill),
	
	imageToggleButtonNew(sortByFileNameButton, 'img/name.png', ''),
	widgetShow(sortByFileNameButton),
	widgetSetTooltip(sortByFileNameButton, 'Sort by file name'),
	boxPackStart(fileSortBox, sortByFileNameButton, expand, fill),
	imageToggleButtonNew(sortByFileSizeButton,'img/size.png', ''),
	widgetSetTooltip(sortByFileSizeButton, 'Sort by file size (biggest file on top)'),
	widgetShow(sortByFileSizeButton),
	boxPackStart(fileSortBox, sortByFileSizeButton, expand, fill),
	imageToggleButtonNew(sortByFileDateButton, 'img/1.png', ''),
	widgetShow(sortByFileDateButton),
	widgetSetTooltip(sortByFileDateButton, 'Sort by file date (most recent file on top)'),
	boxPackStart(fileSortBox, sortByFileDateButton, expand, fill),

	% END

	% REGION file show date/dir/size box
	hBoxNew(fileDateDirSizeBox),
	widgetShow(fileDateDirSizeBox),
	boxPackStart(fileBox, fileDateDirSizeBox, doNotExpand, fill),
	
	labelNew(fileDateDirSizeLabel, 'Show ', noLineWrap),
	widgetShow(fileDateDirSizeLabel),
	boxPackStart(fileDateDirSizeBox, fileDateDirSizeLabel, doNotExpand, fill),
	
	imageToggleButtonNew(showFileDirButton, 'img/dir.png', ''),
	widgetSetTooltip(showFileDirButton, 'Show directories'),
	widgetShow(showFileDirButton),
	boxPackStart(fileDateDirSizeBox, showFileDirButton, expand, fill),
	
	imageToggleButtonNew(showFileSizeButton,'img/size.png', ''),
	widgetSetTooltip(showFileSizeButton, 'Show file sizes'),
	widgetShow(showFileSizeButton),
	boxPackStart(fileDateDirSizeBox, showFileSizeButton, expand, fill),
	
	imageToggleButtonNew(showFileDateButton, 'img/1.png', ''),
	widgetShow(showFileDateButton),
	widgetSetTooltip(showFileDateButton, 'Show file dates'),
	boxPackStart(fileDateDirSizeBox, showFileDateButton, expand, fill),	
	imageToggleButtonNew(showHiddenFilesButton, 'img/hidden.png', ''),
	widgetSetTooltip(showHiddenFilesButton, 'Show hidden files'),
	widgetShow(showHiddenFilesButton),
	boxPackStart(fileDateDirSizeBox, showHiddenFilesButton, expand, fill),

	% END

	widgetShow(fileSortBox),
	% END
*/
	
	% REGION device panel

	% REGION substring box
	hBoxNew(filterHBox(device)),
	%widgetShow(filterHBox(device)),
	boxPackStart(deviceBox, filterHBox(device), doNotExpand, fill),


	imageNew(mbuto(device), 'img/mbuto.png'),
	widgetShow(mbuto(device)),
	boxPackStart(filterHBox(device), mbuto(device), doNotExpand, fill),

	imageButtonNew(deleteSubstringFilterButton(device), 'img/delete-mbuto.png', ''),
	boxPackStart(filterHBox(device), deleteSubstringFilterButton(device), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(device), 'Remove the current substring filter'),

		
	entryNew(containingTextEntry(device)),
	widgetShow(containingTextEntry(device)),
	boxPackStart(filterHBox(device), containingTextEntry(device), expand, fill),
	widgetSetTooltip(containingTextEntry(device),
			 'SUBSTRING SEARCH: If you enter some text here, only devices whose name contains that text will be shown.'),

	imageNew(horizArrow(device), 'img/horiz-arrow.gif'),
	boxPackStart(filterHBox(device), horizArrow(device), doNotExpand, fill),


	% END substring box

	frameNew(deviceFrame),
	boxPackStart(deviceBox, deviceFrame, doNotExpand, fill),
	
	hBoxNew(deviceFilterBox),
	widgetShow(deviceFilterBox),
	containerAdd(deviceFrame, deviceFilterBox),
	
	

	imageButtonNew(editFeaturesButton(device), 'img/edit-info.png', ''),
	buttonSetReliefNone(editFeaturesButton(device)),
	widgetShow(editFeaturesButton(device)),
	boxPackStart(deviceFilterBox, editFeaturesButton(device), expand, fill),
	widgetSetTooltip(editFeaturesButton(device), 'DISPATCH DEVICE: Click here to catalog the selected device'),
	widgetDisable(editFeaturesButton(device)),

	imageButtonNew(newFilterButton(device),'img/filter.png',  ''),
	buttonSetReliefNone(newFilterButton(device)),
	widgetShow(newFilterButton(device)),
	widgetSetTooltip(newFilterButton(device), 'CHANGE DEVICE VIEW: Click here to set which devices must be visible and which must be hidden'),
	boxPackStart(deviceFilterBox, newFilterButton(device), expand, fill),

	imageButtonNew(recentFiltersButton(device),  'img/1.png', ''),
	buttonSetReliefNone(recentFiltersButton(device)),
	widgetSetTooltip(recentFiltersButton(device), 'RECENT VIEWS: Quickly get back to a previous view of devices'),
	%widgetShow(recentFiltersButton(device)),
	boxPackStart(deviceFilterBox, recentFiltersButton(device), expand, fill),


	hBoxNew(labelAndArrowBox(device)),
	widgetShow(labelAndArrowBox(device)),
	boxPackStart(deviceBox, labelAndArrowBox(device), doNotExpand, fill),

	imageNew(arrowImage(device), 'img/redarrow.gif'),
	boxPackStart(labelAndArrowBox(device), arrowImage(device), doNotExpand, fill),
	
	eventLabelNew(labelCurrentDeviceFilter, 'All devices', lineWrap),
	widgetSetTooltip(labelCurrentDeviceFilter, 'This label reminds you which devices are currently visible. You can change this with the Change-view button.'),
	widgetShow(labelCurrentDeviceFilter),
	boxPackStart(labelAndArrowBox(device), labelCurrentDeviceFilter, expand, fill),
	

	checkListBoxNew(checkListBoxOfKind(device), relief, whiteBackground),
	widgetShow(checkListBoxOfKind(device)),
	boxPackStart(deviceBox, checkListBoxOfKind(device), expand, fill),

	curTable(device, TableDevice),
	findall(D, t(TableDevice, device,D), Devices),
	sortByNameInsensitive(Devices, Devices2),
	forall(member(D, Devices2),
	       addWithProperIcon(device, checkListBoxOfKind(device), D)),

	
	% END
	% REGION program panel
	writeln(program-panel),
	% REGION substring box
	hBoxNew(filterHBox(program)),
	%widgetShow(filterHBox(program)),
	boxPackStart(programBox, filterHBox(program), doNotExpand, fill),


	imageNew(mbuto(program), 'img/mbuto.png'),
	widgetShow(mbuto(program)),
	boxPackStart(filterHBox(program), mbuto(program), doNotExpand, fill),

	imageButtonNew(deleteSubstringFilterButton(program), 'img/delete-mbuto.png', ''),
	boxPackStart(filterHBox(program), deleteSubstringFilterButton(program), doNotExpand, fill),
	widgetSetTooltip(deleteSubstringFilterButton(program), 'Remove the current substring filter'),

	entryNew(containingTextEntry(program)),
	widgetShow(containingTextEntry(program)),
	boxPackStart(filterHBox(program), containingTextEntry(program), expand, fill),
	widgetSetTooltip(containingTextEntry(program),
			 'SUBSTRING SEARCH: If you enter some text here, only programs whose name contains that text will be shown.'),

	imageNew(horizArrow(program), 'img/horiz-arrow.gif'),
	boxPackStart(filterHBox(program), horizArrow(program), doNotExpand, fill),

	% END substring box

	frameNew(programFrame),
	boxPackStart(programBox, programFrame, doNotExpand, fill),
	
	hBoxNew(programFilterBox),
	widgetShow(programFilterBox),
	containerAdd(programFrame, programFilterBox),

	%hBoxNew(programFilterBox),
	%widgetShow(programFilterBox),
	%boxPackStart(programBox, programFilterBox, doNotExpand, fill),

	imageButtonNew(editFeaturesButton(program), 'img/edit-info.png', ''),
	buttonSetReliefNone(editFeaturesButton(program)),
	widgetShow(editFeaturesButton(program)),
	widgetSetTooltip(editFeaturesButton(program), 'DISPATCH PROGRAM: Click here to catalog the selected program'),
	boxPackStart(programFilterBox, editFeaturesButton(program), expand, fill),
	widgetDisable(editFeaturesButton(program)),

	imageButtonNew(newFilterButton(program), 'img/filter.png', ''),
	buttonSetReliefNone(newFilterButton(program)),
	widgetSetTooltip(newFilterButton(program), 'CHANGE PROGRAM VIEW: Click here to set which programs must be visible and which must be hidden.'),
	widgetShow(newFilterButton(program)),
	boxPackStart(programFilterBox, newFilterButton(program), expand, fill),

	imageButtonNew(recentFiltersButton(program), 'img/1.png', ''),
	buttonSetReliefNone(recentFiltersButton(program)),
	%widgetShow(recentFiltersButton(program)),
	widgetSetTooltip(recentFiltersButton(program), 'RECENT VIEWS: Quickly get back to a previous view of programs'),
	boxPackStart(programFilterBox, recentFiltersButton(program), expand, fill),


	hBoxNew(labelAndArrowBox(program)),
	widgetShow(labelAndArrowBox(program)),
	boxPackStart(programBox, labelAndArrowBox(program), doNotExpand, fill),

	imageNew(arrowImage(program), 'img/redarrow.gif'),
	boxPackStart(labelAndArrowBox(program), arrowImage(program), doNotExpand, fill),
	
	eventLabelNew(labelCurrentProgramFilter, 'All programs', lineWrap),
	widgetSetTooltip(labelCurrentProgramFilter, 'This label reminds you which programs are currently visible. You can decide which programs you want to be visible with the Change-view button.'),
	widgetShow(labelCurrentProgramFilter),
	boxPackStart(labelAndArrowBox(program), labelCurrentProgramFilter, expand, fill),


	checkListBoxNew(checkListBoxOfKind(program), relief, whiteBackground),
	boxPackStart(programBox, checkListBoxOfKind(program), expand, fill),
	
	curTable(program, TableProg),
	findall(P, t(TableProg,program, P), Programs),
	sortByNameInsensitive(Programs, Programs2),
	forall(member(M, Programs2),
	       addWithProperIcon(program, checkListBoxOfKind(program), M)),
	widgetShow(checkListBoxOfKind(program)),

	
	% END
	% REGION time panel
	writeln(building-time-panel),

	hBoxNew(labelAndArrowBox(time)),
	widgetShow(labelAndArrowBox(time)),
	boxPackStart(deviceBox, labelAndArrowBox(time), doNotExpand, fill),

	imageNew(arrowImage(time), 'img/redarrow.gif'),
	boxPackStart(labelAndArrowBox(time), arrowImage(time), doNotExpand, fill),
	
	labelNew(labelCurrentTimeFilter, 'All times', lineWrap),
	widgetShow(labelCurrentTimeFilter),
	boxPackStart(labelAndArrowBox(time), labelCurrentTimeFilter, expand, fill),
	
	checkListBoxNew(checkListBoxOfKind(time), relief, whiteBackground),
	widgetShow(checkListBoxOfKind(time)),
	boxPackStart(deviceBox, checkListBoxOfKind(time), expand, fill),
	checkListBoxAddItem(checkListBoxOfKind(time), 'Now', 'Now', '', '', '', 'img/1.png', buttonInvisible),
	checkListBoxAddItem(checkListBoxOfKind(time), '15 seconds from now', '15 seconds from now', '', '', '', 'img/1.png', buttonInvisible),
	checkListBoxAddItem(checkListBoxOfKind(time), 'Every 32 minutes starting from now', 'every 32 minutes starting from now',
				    '', '', '', 'img/1.png', buttonInvisible),

		% REGION time button box
	hBoxNew(timeFilterBox),
	widgetShow(timeFilterBox),
	boxPackStart(deviceBox, timeFilterBox, doNotExpand, fill),
	
	imageButtonNew(editTimeFeaturesButton, 'img/1.png', 'Add'),
	buttonSetReliefNone(editTimeFeaturesButton),
	widgetShow(editTimeFeaturesButton),
	widgetSetTooltip(editTimeFeaturesButton, 'Click here to create a new time.'),
	boxPackStart(timeFilterBox, editTimeFeaturesButton, expand, fill),

	imageButtonNew(deleteTimeButton, 'img/remove-time.png', 'Remove'),
	buttonSetReliefNone(deleteTimeButton),
	widgetShow(deleteTimeButton),
	widgetSetTooltip(deleteTimeButton, 'Delete the selected times.'),
	boxPackStart(timeFilterBox, deleteTimeButton, expand, fill),
	% END

	% END
	% REGION log
	writeln(creating-log),
	textViewNew(logTextView),
	%widgetShow(logTextView),
	boxPackStart(mainBoxVertical, logTextView, doNotExpand, fill),
	% END
		% REGION toolbar with option and about button
	hBoxNew(optionsToolbarBox),
	widgetShow(optionsToolbarBox),
	boxPackStart(mainBoxVertical, optionsToolbarBox, doNotExpand, fill),


	vBoxNew(fakeBox),
	widgetShow(fakeBox),
	widgetSetSizeRequest(fakeBox, 2,2),
	boxPackStart(optionsToolbarBox, fakeBox, expand, doNotFill),
	

%	imageToggleButtonNew(advancedButton, 'img/icon.png', 'Show advanced interface'),
%	buttonSetReliefNone(advancedButton),
%	widgetSetTooltip(advancedButton, 'If you click here, additional buttons will appear allowing you to catalog verbs, devices and programs.'),
%	widgetShow(advancedButton),
%	boxPackEnd(optionsToolbarBox, advancedButton, doNotExpand, doNotFill),
	
%	imageToggleButtonNew(showLogButton, 'img/text-file.png', 'Log'),
%	buttonSetReliefNone(showLogButton),
%	widgetSetTooltip(showLogButton, 'If you click here, a log containing the list of actions executed in the past will appear.'),
%	widgetShow(showLogButton),
%	boxPackEnd(optionsToolbarBox, showLogButton, doNotExpand, doNotFill),

	
%	imageButtonNew(manageTasksButton, 'img/1.png', 'Scheduled tasks'),
%	buttonSetReliefNone(manageTasksButton),
%	widgetSetTooltip(manageTasksButton, 'If you click here, a window will appear containing the tasks you have scheduled to be executed in the future. You can cancel tasks which have not been executed yet.'),
%	widgetShow(manageTasksButton),
%	boxPackEnd(optionsToolbarBox, manageTasksButton, doNotExpand, doNotFill),



%	imageToggleButtonNew(showTooltipsButton, 'img/name.png', 'Show tooltips'),
%	buttonSetReliefNone(showTooltipsButton),
%	widgetSetTooltip(showTooltipsButton, 'If pop-up messages like this annoy you, you can disable them with this button. Please keep them until you have learned to use segusoLand.'),
%	widgetShow(showTooltipsButton),
%	toggleButtonSetPressedState(showTooltipsButton, pressed),
%	boxPackEnd(optionsToolbarBox, showTooltipsButton, doNotExpand, doNotFill),


%	imageButtonNew(optionsButton, 'img/options.png', 'Options'),
%	buttonSetReliefNone(optionsButton),
%	widgetSetTooltip(optionsButton, 'Click here to enter the options dialog box. Currently you can only set the directories watched by segusoLand.'),
%	widgetShow(optionsButton),
%	boxPackEnd(optionsToolbarBox, optionsButton, doNotExpand, doNotFill),

%	imageButtonNew(aboutButton, 'img/icon.png', 'About'),
%	buttonSetReliefNone(aboutButton),
%	widgetSetTooltip(aboutButton, 'Informations about the author(s)'),
%	widgetShow(aboutButton),
%	boxPackEnd(optionsToolbarBox, aboutButton, doNotExpand, doNotFill),

	%hSeparatorNew(mainSeparator),
	%widgetShow(mainSeparator),
	%boxPackStart(mainBoxVertical, mainSeparator, doNotExpand, fill),
	% END

	writeln(creating-log-end),
        setGoButton,
	windowMaximize(mainWindow),
	widgetShow(mainWindow),
	widgetDestroy(splashScreen),
	mainLoop([]).

mainLoop(Tasks):-
	var(Tasks),
	writeln(error-var-tasks),
	throw(var-tasks).
mainLoop(Tasks):-
	nonvar(Tasks),
	sleepUntilEventIsRaised(E),
	(
  	  E=checkListBoxItemExecuted(checkListBoxOfKind(verb), I)->
	  onVerbExecuted(I, Tasks, NewTasks);
    	  E=checkListBoxItemSelected(checkListBoxOfKind(verb), I)->
	  (onVerbSelected(I), NewTasks = Tasks);
    	  E=checkListBoxItemDeselected(checkListBoxOfKind(verb), I)->
	  (onVerbDeselected(I), NewTasks = Tasks);
	  E = timer->
	    onTimer(Tasks, NewTasks);
	  E = buttonClicked(editTimeFeaturesButton)->
	    (onAddTimeButtonPressed, NewTasks = Tasks);
	  E=checkListBoxItemSelected(checkListBoxOfKind(K), I)->
	  (onObjectSelected(K, I), NewTasks = Tasks);
	  E=checkListBoxItemDeselected(checkListBoxOfKind(K), I)->
	  (onObjectDeselected(K, I), NewTasks = Tasks);

	  E = buttonClicked(deselectAllButton)->
	     ( onDeselectAllButtonPressed, NewTasks = Tasks);
	  E=buttonClicked(sortByFileNameButton)->
	     ( onSortByFileNameButtonToggled, NewTasks = Tasks);
	  E=buttonClicked(sortByFileDateButton)->
	     ( onSortByFileDateButtonToggled, NewTasks = Tasks);
	  E=buttonClicked(sortByFileSizeButton)->
	     ( onSortByFileSizeButtonToggled, NewTasks = Tasks);
	  E= buttonClicked(showHiddenFilesButton)->
	     ( onShowHiddenFilesButtonToggled, NewTasks = Tasks);
	  E=buttonClicked(reparseDiskButton)->
	  (onReparseDiskButtonPressed, NewTasks = Tasks);
	  
%	  E=buttonClicked(showTooltipsButton)->
%	   ( onShowTooltipsButtonToggled, NewTasks = Tasks);
	  E = menuItemToggled(tooltipsItem)->
	   ( onShowTooltipsButtonToggled, NewTasks = Tasks);
	  
	  E=buttonClicked(showFileDateButton)->
	   ( onShowDateToggled, NewTasks = Tasks);
	  E=buttonClicked(showFileDirButton)->
	   ( onShowDirToggled, NewTasks = Tasks);
	  E=buttonClicked(showFileSizeButton)->
	   ( onShowSizeToggled, NewTasks = Tasks);


	  
%	  E=buttonClicked(aboutButton)->
%	   ( onAboutButtonPressed, NewTasks = Tasks);
	  E = menuItemClicked(aboutItem)->
	  ( onAboutButtonPressed, NewTasks = Tasks);
%	  E=buttonClicked(showLogButton)->
%	   ( onShowLogButtonPressed, NewTasks = Tasks);
	  E = menuItemToggled(logItem)->
	   ( onShowLogButtonPressed, NewTasks = Tasks);

	  %E=buttonClicked(advancedButton)->
	  % ( onAdvancedButtonToggled , NewTasks = Tasks);
	  E=menuItemToggled(advancedItem)->
	   ( onAdvancedItemToggled , NewTasks = Tasks);

	  
%	  E=buttonClicked(optionsButton)->
%	   ( onOptionsButtonPressed, NewTasks = Tasks);
	  E = menuItemClicked(directoriesItem)->
	   ( onOptionsButtonPressed, NewTasks = Tasks);

	  E=buttonClicked(newFilterButton(K))->
	   ( onNewFilterButtonPressedAux(K), NewTasks = Tasks);

	  
	  E=buttonClicked(editFeaturesButton(K))->
	   ( onEditFeaturesButtonPressedAux(K)	, NewTasks = Tasks);

	  E=buttonClicked(showQuickSearchButton)->
	   ( onShowQuickSearchButtonToggled, NewTasks = Tasks);
	  
	  %E = buttonClicked(manageTasksButton)->
	  %onManageTasksClicked(Tasks, NewTasks);

	  E = menuItemClicked(tasksItem)->
	  onManageTasksClicked(Tasks, NewTasks);
	  
	  E = buttonClicked(deleteSubstringFilterButton(K2))->
	    (entrySetText(containingTextEntry(K2), ''), onContainingTextEntryTextChanged(K2), NewTasks = Tasks);
	  E = entryTextChanged(containingTextEntry(K2))->
	     (onContainingTextEntryTextChanged(K2), NewTasks = Tasks);
	  E=buttonClicked(recentFiltersButton(K))->
	   ( onRecentFiltersButtonPressedAux(K)	, NewTasks = Tasks);

	  E=buttonClicked(recentViewsButtonNewbie)->
	  	   ( onRecentFiltersButtonPressedAux(file), NewTasks = Tasks);
	  E=buttonClicked(changeViewButtonNewbie)->
	  	   ( onNewFilterButtonPressedAux(file), NewTasks = Tasks);
	  E=buttonClicked(dispatchButtonNewbie)->
	  	   ( onEditFeaturesButtonPressedAux(file), NewTasks = Tasks);
          E=menuItemClicked(quitItem)->
                  (writeln(menuItemClicked(quitItem)), widgetDestroy(mainWindow), onMainWindowDestroyed, NewTasks = Tasks);
	  E=windowDestroyed(mainWindow)->
	   (  onMainWindowDestroyed, NewTasks = Tasks);
	  (writeln(ignoredEvent(E)), NewTasks = Tasks)),
	
	(E=windowDestroyed(mainWindow)->
	    writeln('exiting program');
	E = menuItemClicked(quitItem)->
	    writeln('exiting program');
	mainLoop(NewTasks)).

onShowLogButtonPressed:-
	%imageToggleButtonIsPressed(showLogButton),
	menuCheckItemIsChecked(viewMenu, logItem),
	widgetShow(logTextView).
onShowLogButtonPressed:-
	%\+imageToggleButtonIsPressed(showLogButton),
	\+menuCheckItemIsChecked(viewMenu, logItem),
	widgetHide(logTextView).
onAdvancedItemToggled:-
	%imageToggleButtonIsPressed(advancedButton),
	menuCheckItemIsChecked(viewMenu, advancedItem),
	widgetShow(verbFrame),
	widgetShow(fileFrame),
	widgetShow(deviceFrame),
	widgetShow(programFrame),
	widgetHide(refreshSeparator),
	widgetShow(recentFiltersButton(verb)),
	widgetShow(recentFiltersButton(device)),
	widgetShow(recentFiltersButton(file)),
	widgetShow(recentFiltersButton(program)),
	widgetHide(dispatchButtonNewbie),
	widgetHide(changeViewButtonNewbie),
	widgetHide(recentViewsButtonNewbie).

%	widgetShow(fileSep),
%	widgetShow(verbSep),
%	widgetShow(deviceSep).
onAdvancedItemToggled:-
	\+menuCheckItemIsChecked(viewMenu, advancedItem),
	%\+imageToggleButtonIsPressed(advancedButton),
	widgetHide(verbFrame),
	widgetHide(fileFrame),
	widgetHide(recentFiltersButton(verb)),
	widgetHide(recentFiltersButton(device)),
	widgetHide(recentFiltersButton(file)),
	widgetHide(recentFiltersButton(program)),
	widgetHide(deviceFrame),
	widgetHide(programFrame),

	widgetShow(refreshSeparator),
	widgetShow(dispatchButtonNewbie),
	widgetShow(changeViewButtonNewbie),
	widgetShow(recentViewsButtonNewbie).

%	widgetHide(fileSep),
%	widgetHide(verbSep),
%	widgetHide(deviceSep).


onShowQuickSearchButtonToggled:-
	imageToggleButtonIsPressed(showQuickSearchButton),
	widgetShow(filterHBox(verb)),
	widgetShow(filterHBox(file)),
	widgetShow(filterHBox(program)),
	widgetShow(filterHBox(device)).
onShowQuickSearchButtonToggled:-
	\+imageToggleButtonIsPressed(showQuickSearchButton),
	widgetHide(filterHBox(verb)),
	widgetHide(filterHBox(file)),
	widgetHide(filterHBox(program)),
	widgetHide(filterHBox(device)).
onAddTimeButtonPressed:-
	dialogNew(newTimeDialog, mainWindow, 'Create new time', 'img/1.png'),

	vBoxNew(newTimeVBox),
	widgetShow(newTimeVBox),
	dialogAddToVBox(newTimeDialog, newTimeVBox),

	labelNew(startTimeLabel, 'Starting time:', noLineWrap),
	widgetShow(startTimeLabel),
	boxPackStart(newTimeVBox, startTimeLabel, expand, fill),

	hBoxNew(startTimeHBox),
	widgetShow(startTimeHBox),
	boxPackStart(newTimeVBox, startTimeHBox, expand, fill),

	% REGION seconds
	labelNew(startLabel(seconds), 'seconds ', noLineWrap),
	widgetShow(startLabel(seconds)),
	boxPackStart(startTimeHBox, startLabel(seconds), doNotExpand, fill),
	
	entryNew(startEntry(seconds)),
	widgetShow(startEntry(seconds)),
	entrySetText(startEntry(seconds), '0'),
	boxPackStart(startTimeHBox, startEntry(seconds), expand, fill),

	% END
	% REGION minutes
	labelNew(startLabel(minutes), 'minutes ', noLineWrap),
	widgetShow(startLabel(minutes)),
	boxPackStart(startTimeHBox, startLabel(minutes), doNotExpand, fill),
	
	entryNew(startEntry(minutes)),
	widgetShow(startEntry(minutes)),
	entrySetText(startEntry(minutes), '0'),
	boxPackStart(startTimeHBox, startEntry(minutes), expand, fill),

	% END
	% REGION hours
	labelNew(startLabel(hours), 'hours ', noLineWrap),
	widgetShow(startLabel(hours)),
	boxPackStart(startTimeHBox, startLabel(hours), doNotExpand, fill),
	
	entryNew(startEntry(hours)),
	widgetShow(startEntry(hours)),
	entrySetText(startEntry(hours), '0'),
	boxPackStart(startTimeHBox, startEntry(hours), expand, fill),

	% END
	% REGION days
	labelNew(startLabel(days), 'days ', noLineWrap),
	widgetShow(startLabel(days)),
	boxPackStart(startTimeHBox, startLabel(days), doNotExpand, fill),
	
	entryNew(startEntry(days)),
	widgetShow(startEntry(days)),
	entrySetText(startEntry(days), '0'),
	boxPackStart(startTimeHBox, startEntry(days), expand, fill),
	% END



	
     	labelNew(periodicTimeLabel, 'Repeat every:', noLineWrap),
	widgetShow(periodicTimeLabel),
	boxPackStart(newTimeVBox, periodicTimeLabel, expand, fill),

	hBoxNew(repeatTimeHBox),
	widgetShow(repeatTimeHBox),
	boxPackStart(newTimeVBox, repeatTimeHBox, expand, fill),

	% REGION seconds
	labelNew(repeatLabel(seconds), 'seconds ', noLineWrap),
	widgetShow(repeatLabel(seconds)),
	boxPackStart(repeatTimeHBox, repeatLabel(seconds), doNotExpand, fill),
	
	entryNew(repeatEntry(seconds)),
	widgetShow(repeatEntry(seconds)),
	entrySetText(repeatEntry(seconds), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(seconds), expand, fill),

	% END
	% REGION minutes
	labelNew(repeatLabel(minutes), 'minutes ', noLineWrap),
	widgetShow(repeatLabel(minutes)),
	boxPackStart(repeatTimeHBox, repeatLabel(minutes), doNotExpand, fill),
	
	entryNew(repeatEntry(minutes)),
	widgetShow(repeatEntry(minutes)),
	entrySetText(repeatEntry(minutes), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(minutes), expand, fill),

	% END
	% REGION hours
	labelNew(repeatLabel(hours), 'hours ', noLineWrap),
	widgetShow(repeatLabel(hours)),
	boxPackStart(repeatTimeHBox, repeatLabel(hours), doNotExpand, fill),
	
	entryNew(repeatEntry(hours)),
	widgetShow(repeatEntry(hours)),
	entrySetText(repeatEntry(hours), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(hours), expand, fill),

	% END
	% REGION days
	labelNew(repeatLabel(days), 'days ', noLineWrap),
	widgetShow(repeatLabel(days)),
	boxPackStart(repeatTimeHBox, repeatLabel(days), doNotExpand, fill),
	
	entryNew(repeatEntry(days)),
	widgetShow(repeatEntry(days)),
	entrySetText(repeatEntry(days), '0'),
	boxPackStart(repeatTimeHBox, repeatEntry(days), expand, fill),

	% END

	
	hBoxNew(createTimeHBox),
	widgetShow(createTimeHBox),
	boxPackStart(newTimeVBox, createTimeHBox, expand, fill),
	
	buttonNew(createTimeButton,'create'),
	widgetShow(createTimeButton),
	widgetDisable(createTimeButton),
	boxPackStart(createTimeHBox, createTimeButton, expand, doNotFill),
	
	widgetShow(newTimeDialog),

	newTimeLoop.

newTimeLoop:-
	sleepUntilEventIsRaised(E),

	( E = entryTextChanged(startLabel(_T))->
	    (onStartTimeChanged, newTimeLoop);
	  E = dialogDestroyed(newTimeDialog)->
	    writeln(exiting-new-time-dialog);
	  (writeln(ignoredEvent(E)), newTimeLoop)).

onContainingTextEntryTextChanged(K):-
	entryGetText(containingTextEntry(K), T),
	T='',
	widgetHide(horizArrow(K)),
	widgetShow(mbuto(K)),
	updateItemsAfterSelectionChanged(K),
	widgetHide(deleteSubstringFilterButton(K)).
onContainingTextEntryTextChanged(K):-
	entryGetText(containingTextEntry(K), T),
	T\='',
	widgetShow(horizArrow(K)),
	widgetHide(mbuto(K)),
	updateItemsAfterSelectionChanged(K),
	widgetShow(deleteSubstringFilterButton(K)).

	
% @@ update view.
/*
	entryGetText(containingTextEntry(file), ''),
	retractall(nameFilter(K, _)).
onContainingTextEntryTextChanged(K):-
	entryGetText(containingTextEntry(file), T),
	T\='',
	retractall(nameFilter(K, _)),
	assert(nameFilter(K, T)).
*/
	
onAboutButtonPressed:-
	dialogNew(aboutDialog, mainWindow, 'About segusoLand', 'img/icon.png'),

	hBoxNew(aboutHBox),
	widgetShow(aboutHBox),
	dialogAddToVBox(aboutDialog, aboutHBox),

	imageNew(candleImage, 'img/big-candle.gif'),
	widgetShow(candleImage),
	boxPackStart(aboutHBox, candleImage, doNotExpand, fill),

	labelNew(aboutLabel, 'segusoLand was created by Maurizio Colucci (seguso.forever@tin.it). \nHome page: http://segusoland.sourceforge.net', lineWrap),
	widgetShow(aboutLabel),
	boxPackStart(aboutHBox, aboutLabel, expand, fill),
	
	widgetShow(aboutDialog).
	
onManageTasksClicked(Tasks, NewTasks):-
	dialogNew(tasksDialog, mainWindow, 'Manage tasks', 'img/1.png'),

	vBoxNew(tasksVBox),
	widgetShow(tasksVBox),
	dialogAddToVBox(tasksDialog, tasksVBox),

	hBoxNew(tasksHbox),
	widgetShow(tasksHbox),
	boxPackStart(tasksVBox, tasksHbox, doNotExpand, fill),

	labelNew(tasksLabel, 'Here you can select some tasks and click \"Delete selected tasks\". If you don\'t want to delete any task, simply close the window.', lineWrap),
	widgetShow(tasksLabel),
	boxPackStart(tasksHbox, tasksLabel, expand, fill),
	
	buttonNew(deleteTaskButton, 'Delete selected tasks'),
	widgetShow(deleteTaskButton),
	boxPackStart(tasksHbox, deleteTaskButton, doNotExpand, doNotFill),
	
	checkListBoxNew(tasksListBox, relief, whiteBackground),
	widgetShow(tasksListBox),
	boxPackStart(tasksVBox, tasksListBox, expand, fill),


	forall(member(T, Tasks),
	       (readableRepresentationOfTask(T, Read),
		   checkListBoxAddItem(tasksListBox, T, Read, '', '', '', 'img/1.png', buttonInvisible))),
	widgetShow(tasksDialog),
	windowMaximize(tasksDialog),
	tasksLoop([], Tasks, NewTasks).

tasksLoop(SelectedTasks, _, _):-
	var(SelectedTasks),
	throw(error-var-selected-tasks).
tasksLoop(_, Tasks, _):-
	var(Tasks),
	throw(error-var-tasks).
tasksLoop(/*in*/SelectedTasks, /*in*/Tasks, /*out*/NewTasks):-
	nonvar(SelectedTasks),
	nonvar(Tasks),
	sleepUntilEventIsRaised(E),
	( E = dialogDestroyed(tasksDialog)->
	    NewTasks = Tasks /*and exit*/;
	  E = buttonClicked(deleteTaskButton)->
	    (writeln(sel-SelectedTasks),
		onDeleteTaskPressed(SelectedTasks, NewSelectedTasks, Tasks, NewTasks2),
		tasksLoop(NewSelectedTasks, NewTasks2, NewTasks));
	  E = checkListBoxItemSelected(tasksListBox, I)->
	    tasksLoop([I|SelectedTasks], Tasks, NewTasks);
	  E = checkListBoxItemDeselected(tasksListBox, I)->
	    (remove( I, SelectedTasks, NewSelectedTasks),
		tasksLoop(NewSelectedTasks, Tasks, NewTasks));
	  (writeln(ignoredEvent(E)),
	      tasksLoop(SelectedTasks, Tasks, NewTasks))).
readableRepresentationOfTask(T, Repr):-
	T=(Time, A),
	convert_time(Time, String),
	Repr = (String, A).
onDeleteTaskPressed(SelectedTasks, []/*NewSelectedTasks*/, Tasks, NewTasks):-
	forall(member(T, SelectedTasks),
	       checkListBoxDeleteItem(tasksListBox, T)),
	removeListFromList(SelectedTasks, Tasks, NewTasks).



removeListFromList([], L2, L2).
removeListFromList([H|T], L2, NewL2):-
	remove(H, L2, NewL2Temp),
	removeListFromList(T, NewL2Temp, NewL2).
onReparseDiskButtonPressed:-
	% @@ ERROR if I delete a file and reparse.
	% REGION 
	forall(t('All files', file, F),
	       (exists_file(F)->
		   true;
		   (retract(t('All files', file, F)),
		       retractall(t('Files to dispatch', file, F)),
		       retractall(ext(_, F)),
		       retractall(hiddenFile(F)),
		       (selected(file, F)-> (retract(selected(file, F)),
						updateItemsAfterSelectionChanged(file))),
		       checkListBoxDeleteItem(checkListBoxOfKind(file), F)))),
	% END
	forall(segusoDir(D),
	       seeIfThereAreNewFilesInDir(D)),
	addStringToLog('Reparsed directories.'),
	% REGION sorting is not correct anymore because any file has been added at the end of the list
	(imageToggleButtonIsPressed(sortByFileNameButton)->toggleButtonSetPressedState(sortByFileNameButton, released); true),
	(imageToggleButtonIsPressed(sortByFileSizeButton)->toggleButtonSetPressedState(sortByFileSizeButton, released); true),
	(imageToggleButtonIsPressed(sortByFileDateButton)->toggleButtonSetPressedState(sortByFileDateButton, released); true).
	% END


addStringToLog(S):-
	% REGION log
	get_time(Time),
	convert_time(Time, TimeString),
	concat_atom([TimeString, ': ', S], Text),
	textViewAddLine(logTextView, Text).
	% END
	
seeIfThereAreNewFilesInDir(D):-
	addSlashToDirIfNeeded(D, DirSlash),
	concat_atom([DirSlash, '*'], ReqVisible),  
	concat_atom([DirSlash, '.*'], ReqHidden), 
	expand_file_name(ReqVisible, VisibleFiles),  % if the dir does not exist, it returns [], which is good! @@ but add a check?
	expand_file_name(ReqHidden, HiddenFiles),
	append(VisibleFiles, HiddenFiles, AllFiles),
	forall((member(F, AllFiles), exists_file(F)),
	       ifFileIsNotAlreadyInDatabaseAddToDatabase(F)).
ifFileIsNotAlreadyInDatabaseAddToDatabase(F):-
	t('All files', file, F).
ifFileIsNotAlreadyInDatabaseAddToDatabase(F):-
	\+t('All files', file, F),
	assert(t('All files', file, F)),
	assertExtensionIfAny(F),
	addWithProperIcon(file, checkListBoxOfKind(file), F).
        % @@@ hidden Files...

	
onShowTooltipsButtonToggled:-
	%imageToggleButtonIsPressed(showTooltipsButton),
	menuCheckItemIsChecked(viewMenu, tooltipsItem),
	setTooltipsState(enabled).
onShowTooltipsButtonToggled:-
	%\+imageToggleButtonIsPressed(showTooltipsButton),
	\+menuCheckItemIsChecked(viewMenu, tooltipsItem),
	setTooltipsState(disabled).
	
% Also works if the Dir does not exist. :-)
addFilesOfDirToAllFilesTable_HiddenFilesTable_ExtTable(Dir):-
	addSlashToDirIfNeeded(Dir, DirSlash),
	concat_atom([DirSlash, '*'], ReqVisible),  
	concat_atom([DirSlash, '.*'], ReqHidden), 
	expand_file_name(ReqVisible, VisibleFiles),  % if the dir does not exist, it returns [], which is good! @@ but add a check?
	expand_file_name(ReqHidden, HiddenFiles),
	% REGIOn hidden table
	forall(member(M, HiddenFiles),
	       (exists_file(M)->assert(hiddenFile(M)); true)),
	% END
	append(VisibleFiles, HiddenFiles, AllFiles),
	% REGION all files table
	forall(member(Q, AllFiles),
	       (exists_file(Q)->assert(t('All files', file, Q)); true)),
	% END
	% REGION see if they are text files
	%forall(member(Z, AllFiles),
	%       assertThisIsATextFileIfItIs(Z)).
	% END
	% REGION fill the extension tables: png/1, avi/1, etc.
	forall(member(Q, AllFiles), assertExtensionIfAny(Q)).
	% END

	
addSlashToDirIfNeeded(Dir, Dir):-
	dirEndsInSlash(Dir).
addSlashToDirIfNeeded(Dir, DirSlash):-
	\+dirEndsInSlash(Dir),
	concat_atom([Dir, '/'], DirSlash).
dirEndsInSlash(Dir):-
	atom_chars(Dir, List),
	listEndsWith(List, '/').
listEndsWith([C], C).
listEndsWith([Element], C):-
	Element\=C,
	fail.
listEndsWith(L, C):-
	L\=[_],
	L=[H|T],
	H\=[], T\=[],
	listEndsWith(T, C).
	
% speed it up!
assertThisIsATextFileIfItIs(F):-
	concat_atom(['file \'', F, '\''], Command),
	open(pipe(Command), read, Stream),
	get(Stream, Term),
	writeln(justRead(Term)),
	close(Stream).

onShowHiddenFilesButtonToggled:-
	imageToggleButtonIsPressed(showHiddenFilesButton),
	assert(showHiddenFiles), % read by commonShit
	commonShit(file).
onShowHiddenFilesButtonToggled:-
	\+imageToggleButtonIsPressed(showHiddenFilesButton),
	retract(showHiddenFiles), % read by commonShit
	commonShit(file).

onShowDirToggled:-
	imageToggleButtonIsPressed(showFileDirButton),
	checkListBoxSetDirVisibility(checkListBoxOfKind(file), visible).
onShowDirToggled:-
	\+imageToggleButtonIsPressed(showFileDirButton),
	checkListBoxSetDirVisibility(checkListBoxOfKind(file), hidden).
onShowDateToggled:-
	imageToggleButtonIsPressed(showFileDateButton),
	checkListBoxSetDateVisibility(checkListBoxOfKind(file), visible).
onShowDateToggled:-
	\+imageToggleButtonIsPressed(showFileDateButton),
	checkListBoxSetDateVisibility(checkListBoxOfKind(file), hidden).
onShowSizeToggled:-
	imageToggleButtonIsPressed(showFileSizeButton),
	checkListBoxSetSizeVisibility(checkListBoxOfKind(file), visible).
onShowSizeToggled:-
	\+imageToggleButtonIsPressed(showFileSizeButton),
	checkListBoxSetSizeVisibility(checkListBoxOfKind(file), hidden).
	
fromKindToCurrentViewLabel(program, labelCurrentProgramFilter).
fromKindToCurrentViewLabel(verb, labelCurrentVerbFilter).
fromKindToCurrentViewLabel(device, labelCurrentDeviceFilter).
fromKindToCurrentViewLabel(file, labelCurrentFileFilter).


createDbIfItDoesNotExist:-
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/segusoLand-database'], DatabaseFileNamewithFullPath),
	exists_file(DatabaseFileNamewithFullPath).
createDbIfItDoesNotExist:-
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/segusoLand-database'], DatabaseFileNamewithFullPath),
	\+exists_file(DatabaseFileNamewithFullPath),
	shell('touch ~/segusoLand-database', Res),
	(Res=0->writeln('WARNING: Database not found, created anew');
	    writeln('could not create database! Exiting')),
	Res=0,
	assert(segusoDir('~')).
	% REGION add a default categorizazion of programs and verbs
        % @@
	% END
	
readLoop(Db):-
	read(Db, T),
	readLoopAux(Db, T).

readLoopAux(_, end_of_file):-
	writeln(end_of_file).
readLoopAux(Db, T):-
	T=table(K, N, real),
	assertIfNotAlreadyThere(table(K, N, real)),  % bug fixed: it used to fail when  assertIfNotAlreadyThere did retract and assert immediately after @@
	% REGION read next
	read(Db, T2),
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=belongsTo(Table, N, Kind),
	assertObjectBelongsToTable(N, Kind, Table),
	% REGION read next
	read(Db, T2),
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=segusoDir(D),
	expand_file_name(D, [DExp]),
	exists_directory(DExp),
	assert(segusoDir(DExp)),
	% REGION read next
	read(Db, T2),
	readLoopAux(Db, T2).
        % END
readLoopAux(Db, T):-
	T=segusoDir(D),
	expand_file_name(D, [DExp]),
	\+exists_directory(DExp),
	concat_atom(['WARNING: Reading database: Discarded directory ', DExp, ' because it does not seem to exist.'], Text),
	writeln(Text),
	% REGION read next
	read(Db, T2),
	readLoopAux(Db, T2).
        % END

readLoopAux(_Db, TermRead):-
	TermRead=curtable(_Kind, _T, _Head, _Body).

readLoopAux(_, T):-
	write('TRAGIC FAILURE: did not match: '),
	writeln(T).
	
assertObjectBelongsToTable(_, K, T):-
	predefinedTable(K, _, T),
	writeln('I found an assertion about the content of a predefined table in the database. Ignoring it'(K, T)).
assertObjectBelongsToTable(ObjName, file, _Table):-
	\+exists_file(ObjName),
	concat_atom(['Discarding info about file ', ObjName, ' because it does not exist.'], Text),
	writeln(Text).
assertObjectBelongsToTable(ObjName, file, Table):-
	\+predefinedTable(file, _, Table),
	exists_file(ObjName),
	file_directory_name(ObjName, Dir),
	iAmWatchingTheDir(Dir),
	assertIfNotAlreadyThere(t(Table, file, ObjName)).
assertObjectBelongsToTable(ObjName, file, _Table):-
	exists_file(ObjName),
	file_directory_name(ObjName, Dir),
	\+iAmWatchingTheDir(Dir),
	concat_atom(['Discarding info about file ', ObjName, ' because I am not watching its dir.'], Text),
	writeln(Text).
assertObjectBelongsToTable(ObjName, Kind, Table):-
	Kind\=file,
	\+predefinedTable(Kind, _, Table),
	assertIfNotAlreadyThere(t(Table,Kind, ObjName)).

% difficult because the dir could be ~ and there could be segusoDir('/home/gesucristo/').
% 1) one might have an unexpanded ~, and the other one could have it expanded.
% 2) one might end in /, the other one may not.
% ALGO: 1)  expand them both 2) add slash if needed 3) do a syntactic equality check (unification).
iAmWatchingTheDir(D):-
	expand_file_name(D, [DExp]),
	segusoDir(SegDir),
	expand_file_name(SegDir, [SegDirExp]),
	% REGION add slash if there isn't any
	addSlashToDirIfNeeded(SegDirExp, SegDirExpSlashed),
	addSlashToDirIfNeeded(DExp, DExpSlashed),
	% END
	DExpSlashed = SegDirExpSlashed.
% @@ possible bug: this does not detect equality between /home/mau and /home//mau


assertIfNotAlreadyThere(_, _, _):-
	writeln('unmanaged case'),
	trace.

onMainWindowDestroyed:-
	writeln('onMainWindowDestroyed: saving database'),
	% save database -- write database
	expand_file_name('~', [HomeDir]),
	concat_atom([HomeDir, '/segusoLand-database'], DatabaseFileNamewithFullPath),
	open(DatabaseFileNamewithFullPath, write, DB),
	% REGION write segusodirs
	forall(segusoDir(D), writeWithNewline(DB, segusoDir(D))),
	% END
	% REGION save each table together with its records
	findall(table(Kind, Name, real),
		(table(Kind,Name,real), \+predefinedTable(Kind, _, Name)),
		RealTables),
	forall(member(M, RealTables), dumpTable(M, DB)),
	% END
	close(DB),
	writeln('database saved').


% REGION dumpTable/2
% REGION do not dump predefined table to the file: it is useless, they are defined in the code. 
dumpTable(table(device, 'All devices', real), _).
dumpTable(table(program, 'All programs', real), _).
dumpTable(table(verb, 'All verbs', real), _).
% END

dumpTable(table(K, N, real), Db):-
	writeWithNewline(Db, table(K, N, real)),
	findall(O, t(N,K, O), Items),  % better not use setof, findall helps discovering repetitions
	forall(member(M, Items), writeWithNewline(Db, belongsTo(N, M, K))).
% END

typeToIcon(txt, 'img/text-file.png').
typeToIcon(pdf, 'img/pdf.png').
typeToIcon(rpm, 'img/rpm.png').
typeToIcon(mp3, 'img/audio.png').
typeToIcon(flac, 'img/audio.png').
typeToIcon(ogg, 'img/audio.png').
typeToIcon(wav, 'img/audio.png').
typeToIcon(ogm, 'img/video.png').
typeToIcon(avi, 'img/video.png').
typeToIcon(wma, 'img/audio.png').
typeToIcon(wmv, 'img/video.png').
typeToIcon(asf, 'img/video.png').
typeToIcon(zip, 'img/archive.png').
typeToIcon(rar, 'img/archive.png').
typeToIcon(gz , 'img/archive.png').
typeToIcon(bz2, 'img/archive.png').
typeToIcon(tar, 'img/archive.png').
typeToIcon(arj, 'img/archive.png').
typeToIcon(ace, 'img/archive.png').
typeToIcon(htm, 'img/internet.png').
typeToIcon(html, 'img/internet.png').
typeToIcon(png, 'img/picture.png').
typeToIcon(gif, 'img/picture.png').
typeToIcon(jpg, 'img/picture.png').
typeToIcon(ppm, 'img/picture.png').
typeToIcon(bmp, 'img/picture.png').
typeToIcon(_, 'img/generic.png').
	

verbToIcon('Create new PDF file', 'img/pdf.png').
verbToIcon('View PDF', 'img/filter.png').
verbToIcon('Turn off', 'img/turnoff.png').
verbToIcon('Print picture', 'img/printer.png').
verbToIcon('Print PDF', 'img/printer.png').
verbToIcon('Play movie with subtitle file', 'img/video.png').
verbToIcon('Play video', 'img/video.png').
verbToIcon('Browse the internet', 'img/internet.png').
verbToIcon('View HTML file', 'img/filter.png').
verbToIcon('View DVI file', 'img/filter.png').
verbToIcon('View content of archive', 'img/filter.png').
verbToIcon('Calculate size', 'img/size.png').
verbToIcon('Delete', 'img/delete.png').
verbToIcon('Play audio', 'img/audio.png').
verbToIcon('Enqueue audio for playing', 'img/audio.png').
verbToIcon('Set sound volume', 'img/volume.png').
verbToIcon('Burn audio CD', 'img/cdrom.png').
verbToIcon('Burn data CD', 'img/cdrom.png').
verbToIcon('Burn from ISO', 'img/cdrom.png').
verbToIcon('Burn video CD', 'img/cdrom.png').
verbToIcon('Restore file from trashcan', 'img/restore-trash.png').
verbToIcon('Move to trashcan', 'img/trash.png').
verbToIcon('Empty trashcan', 'img/empty-trash.png').
verbToIcon('Erase CD-RW', 'img/cdrom.png').
verbToIcon('Burn CD from audio and CUE files', 'img/cdrom.png').
verbToIcon('Burn CD from BIN and CUE files', 'img/cdrom.png').
verbToIcon('Add to different archives', 'img/archive.png').
verbToIcon('Add to single archive', 'img/archive.png').
verbToIcon('Extract archive', 'img/archive.png').
verbToIcon('Create new text file', 'img/text-file.png').
verbToIcon('Edit plain text file', 'img/text-file.png').
verbToIcon('Print text file', 'img/printer.png').
verbToIcon('Edit picture file', 'img/picture.png').
verbToIcon('View picture', 'img/filter.png').
verbToIcon('Install RPM', 'img/rpm.png').
verbToIcon('Install DEB', 'img/rpm.png').
verbToIcon('Send to Konqueror', 'img/konqueror.png').
verbToIcon('View calendar and schedule', 'img/schedule.png').
verbToIcon('Check new email', 'img/email.png').
verbToIcon('Setup mouse', 'img/mouse.png').
verbToIcon('Change keyboard layout', 'img/keyboard.png').
verbToIcon('Change resolution', 'img/resolution.png').
verbToIcon('Configure printer', 'img/printer.png').
verbToIcon('View print queue', 'img/printer.png').
verbToIcon(_, 'img/generic.png').

programToIcon('Korganizer', 'img/schedule.png').
programToIcon('Konqueror', 'img/konqueror.png').
programToIcon('K3b', 'img/k3b.png').
programToIcon('Evolution', 'img/evolution.png').
programToIcon('Kmail', 'img/kmail.png').
programToIcon('Mozilla', 'img/mozilla.png').
programToIcon('Mozilla Firebird', 'img/mozilla.png').
programToIcon('Xpdf', 'img/xpdf.png').
programToIcon('Emacs', 'img/emacs.png').
programToIcon('Aumix', 'img/aumix.png').
programToIcon('Kmix', 'img/kmix.png').
programToIcon('Trashcan', 'img/trash.png').
programToIcon('segusoLand', 'img/icon.png').
programToIcon(_, 'img/generic.png').

deviceToIcon('Floppy Drive', 'img/floppy.png').
deviceToIcon('My monitor', 'img/monitor.png').
deviceToIcon('My Keyboard', 'img/keyboard.png').
deviceToIcon('My Mouse', 'img/mouse.png').
deviceToIcon('My printer', 'img/printer.png').
deviceToIcon('My Computer', 'img/computer.png').
deviceToIcon('My sound card', 'img/audio.png').
deviceToIcon('My CD writer', 'img/cdrom.png').
deviceToIcon('My CD reader', 'img/cdrom.png').
deviceToIcon(_, 'img/generic.png').

writeWithNewline(Database, Term):-
	writeq(Database, Term),
	put(Database, '.'),
	put(Database, '\n').

assertExtensionIfAny(B):-
	file_name_extension(_, Ext, B),
	Ext\='',
	nonvar(Ext),
	%verbRequiresAtLeastOneObjectSatisfying(_, ext(Ext)),  % if the extension is useful.... @@ doesn't work because there
	% areindirect extensions, such as videoFile... We could transform videoFile/1 into ext(videoFile, _), and it would work.
	assert(ext(Ext, B)).
assertExtensionIfAny(B):-
	file_name_extension(_, '', B).



/*
% REGION sortByNameInsensitiveNotCountingPath
sortByNameInsensitiveNotCountingPath(List, Sorted):-
	addBasenameInFront(List, List2),
	sort(List2, List3),
	removeBasenameInFront(List3, Sorted).
addBasenameInFront([], []).
addBasenameInFront([H|T], [L-H|T2]):-
	file_base_name(H, B),
	downcase_atom(B, L),
	addBasenameInFront(T, T2).
removeBasenameInFront([], []).
removeBasenameInFront([_-H|T], [H|T2]):-
	removeBasenameInFront(T, T2).
% END
*/


% REGION sortByNameInsensitive
sortByNameInsensitive(List, Sorted):-
	addKeys(List, Keys),
	sort(Keys, S),
	removeKeys(S, Sorted).
addKeys([], []).
addKeys([H|T], [L-H|T1]):-
	downcase_atom(H, L),
	addKeys(T, T1).
removeKeys([], []).
removeKeys([_-H|T], [H|T1]):-
	removeKeys(T, T1).
% END


onSortByFileNameButtonToggled:-
	imageToggleButtonIsPressed(sortByFileNameButton),
	toggleButtonSetPressedState(sortByFileDateButton, released),
	toggleButtonSetPressedState(sortByFileSizeButton, released), 
	retractall(fileSortMode(_)),
	assert(fileSortMode(name)),
	checkListBoxSortByName(checkListBoxOfKind(file)).
onSortByFileNameButtonToggled:-
	\+imageToggleButtonIsPressed(sortByFileNameButton).

onSortByFileDateButtonToggled:-
	imageToggleButtonIsPressed(sortByFileDateButton),
	toggleButtonSetPressedState(sortByFileNameButton, released),
	toggleButtonSetPressedState(sortByFileSizeButton, released),  
	retractall(fileSortMode(_)),
	assert(fileSortMode(date)),
	checkListBoxSortByDate(checkListBoxOfKind(file)).
onSortByFileDateButtonToggled:-
	\+imageToggleButtonIsPressed(sortByFileDateButton).


onSortByFileSizeButtonToggled:-
	imageToggleButtonIsPressed(sortByFileSizeButton),
	toggleButtonSetPressedState(sortByFileNameButton, released),
	toggleButtonSetPressedState(sortByFileDateButton, released),
	retractall(fileSortMode(_)),
	assert(fileSortMode(size)),
	checkListBoxSortBySize(checkListBoxOfKind(file)).
onSortByFileSizeButtonToggled:-
	\+imageToggleButtonIsPressed(sortByFileSizeButton).



/*
sortFilesAccordingToMode(F, Sorted):-
	fileSortMode(name),
	sortByNameInsensitiveNotCountingPath(F, Sorted).
sortFilesAccordingToMode(F, Sorted):-
	fileSortMode(date),
	predsort(sortByDate, F, Sorted).
sortFilesAccordingToMode(F, Sorted):-
	fileSortMode(size),
	predsort(sortBySize, F, Sorted).



onSortFileAux:-
	% REGION empty checklistbox 
	checkListBoxOfKind(file, C),
	% END
	% REGION sort selected objects of that kind (even not in current table)
	findall(O2, selected(file, O2), SelectedFiles),
	sortObjectsAccordingToKindAndMode(file, SelectedFiles, SortedSelectedFiles),
	% END

	% REGION 2) add all the objects in current table which
	             %are not selected (selected ones have already been added)
	curTable(file, T),
	findall(O, (t(T,file, O), \+selected(file, O)), Objs),   % do not use setof or bagof! THey can fail.
	sortFilesAccordingToMode(Objs, SortedFiles).
	% END

	  after sorting  you don't need to do narrowing...
	  	selectedObjects(Sel)
	% REGION narrow verbs
	curTable(verb, TableVerbs),
	forall(t(TableVerbs,verb, V), decideVerbVisibility(V)),
	% END
	% REGION narrow objects to only those objects that are required by a visible verb!
	findall(V, theVerbIsVisible(V, Sel), VisibleVerbs),
	forall(objectKind(K2),
	       (curTable(K2, T2),
		   forall(t(T2,K2, O2), decideObjectVisibility(O2,K2, VisibleVerbs)))).
*/



onRecentFiltersButtonPressedAux(Kind):-
	dialogNew(recentFileFiltersDialog, mainWindow, 'Choose recent view', 'img/1.png'),
	
	vBoxNew(recentFileFiltersBox),
	widgetShow(recentFileFiltersBox),
	dialogAddToVBox(recentFileFiltersDialog, recentFileFiltersBox),
	
	% REGION show current filter in a label
	%findall(X, selectedFilterPair(Kind, X), CurrentFilter),
	%compactTextualRepresentationOfFilter(CurrentFilter, R),
	%concat_atom(['The current view is ', R, '.'], R2),
	labelNew(currentFileFilterLabel, 'Choose recent view', lineWrap),
	widgetShow(currentFileFilterLabel),
	boxPackStart(recentFileFiltersBox, currentFileFilterLabel, doNotExpand, fill),
		
	checkListBoxNew(recentFiltersCheckListBox, noRelief, noWhiteBackground), 
	widgetShow(recentFiltersCheckListBox),
	boxPackStart(recentFileFiltersBox, recentFiltersCheckListBox, expand, fill),

	% REGION fill the dialog with all recent filters
	% possible improvement: @@ EXCEPT the current one, if any
	forall(recentView(Kind, Predicate, ReadableNameOfPredicate),
	       checkListBoxAddItem(recentFiltersCheckListBox,
				   Predicate,
				   ReadableNameOfPredicate,
				   '', '', '', 'img/filter.png',
				   buttonInvisible)),
	
	% END
	windowMaximize(recentFileFiltersDialog),
	widgetShow(recentFileFiltersDialog),
	recentFiltersLoop(Kind).

recentFiltersLoop(K):-
	sleepUntilEventIsRaised(E),
	( E = checkListBoxItemSelected(recentFiltersCheckListBox, I)->
	       onRecentFilterSelected(K, I);
	  E=dialogDestroyed(recentFileFiltersDialog)->
	     true;
	  writeln(ignoredEvent(E))),

	(E = dialogDestroyed(recentFileFiltersDialog)->
	    writeln(exitingDialogLoop);
	 E=checkListBoxItemSelected(recentFiltersCheckListBox, _)->
	    writeln(exitingDialogLoop);
	 recentFiltersLoop(K)).


onRecentFilterSelected(K, Predicate):-
	retractall(curTable(K, _)),
	assert(curTable(K, Predicate)),
	commonShit(K),
	widgetDestroy(recentFileFiltersDialog),
	setGoButton.


% REGION setGoButton  @@ optimize
setGoButton:-
	findall(V2, theGoButtonIsNotVisible(V2), IncompleteVerbs),
	forall(member(M2, IncompleteVerbs), checkListBoxSetButtonVisibility(checkListBoxOfKind(verb), M2, hidden)),
	findall(V, theGoButtonIsVisible(V), CompleteVerbs),
	sort(CompleteVerbs, CompleteVerbsSorted),
	forall(member(M, CompleteVerbsSorted), checkListBoxSetButtonVisibility(checkListBoxOfKind(verb), M, visible)).
theGoButtonIsNotVisible(V):-
	curTable(verb, T),
	t(T,verb, V),
	theVerbIsIncomplete(V).
theGoButtonIsNotVisible(V/*var*/):-
	curTable(verb, T),
	t(T,verb, V),
	selectionIncompatibleWithVerb(V).  % @@ too slow to check for each verb? We could only check whether there is a nonrequired object.

theVerbIsIncomplete(V):-
	selected(time, _),
	verbRequiresAtLeastOneObjectSatisfying(V, K, Pred),
	forall(selected(K, O), \+call(Pred, O)).  
theVerbIsIncomplete(_):-
	\+selected(time, _).

theGoButtonIsVisible(V/*var*/):-
	curTable(verb, T),
	t(T, verb, V),
	theVerbIsComplete(V),
	\+selectionIncompatibleWithVerb(V).

theVerbIsComplete(V):-
	findall( verbRequiresAtLeastOneObjectSatisfying(V, K,P),
		 verbRequiresAtLeastOneObjectSatisfying(V, K, P), Predicates),
	forall(member(verbRequiresAtLeastOneObjectSatisfying(V, K2, P2), Predicates),
	       (selected(K2, O), call(P2, O))),
	selected(time, _).
thereIsASelectedMemberThatSatisfies(P, Sel):-
	member(M, Sel), % @@ do not treat objects generically. refactor code to this purpose.
	call(P, M).

% END


% @@ should never fail... but it fails
assertIfNotAlreadyThere(X):-
	X.
assertIfNotAlreadyThere(X):-
	\+X,
	assert(X).
assertIfNotAlreadyThere(_):-
	writeln('assertIfNotAlreadyThere failed...'),
	throw('tragic error'),
	trace.



addNewlinesToString(Position, % if Position=5, I wrap every 5 letters*/
		    String, % in
		    NewString):-  % out
	atom_chars(String, List),
	addNewlinesToStringAux(Position, 0, [], List, ListWithNewlinesInserted),
	concat_atom(ListWithNewlinesInserted, NewString).
addNewlinesToStringAux(_, _Count, CurList, [], CurList).
addNewlinesToStringAux(Position, Position, CurList, [H|T], NewList):-
	H\=[],
	append(CurList, [H, '\n'], C2),
	addNewlinesToStringAux(Position, 0, C2, T, NewList).
addNewlinesToStringAux(Position, Count, CurList, [H|T], NewList):-
	H\=[],
	Count < Position,
	NewCount is Count+1,
	append(CurList, [H], C2),
	addNewlinesToStringAux(Position, NewCount, C2, T, NewList).
	
:-dynamic(nextFreeConst/1).
nextFreeConst(1).

newconst(C):-
	nextFreeConst(F),
	concat_atom([p, F], C),
	retract(nextFreeConst(F)),
	D is F+1,
	assert(nextFreeConst(D)).

	



presentTime('Now').

periodicTime('every 32 minutes starting from now').
